@@ -2,9 +2,9 @@
 
 /************************************************************************
  *
- * $Id: cd2lb.c,v 1.10 2001/12/26 22:06:09 jon Exp $
+ * $Id: cd2lb.c,v 1.13 2003/11/02 20:53:00 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /**********************************************************************/
@@ -54,9 +54,12 @@
 
 #include "conqdef.h"
 
-#include "conqcom2.h"
+#include "context.h"
+#include "conf.h"
 #include "global.h"
 #include "color.h"
+#include "ibuf.h"
+#include "display.h"
 
 #define MSGMAXLINE 90 			/* used for screen formatting */
 
@@ -215,7 +218,7 @@ void cdend(void)
 
 #if defined (HAVE_TERMIOS_H)
   tcgetattr(PollInputfd, &term);
-  term.c_cc[VINTR] = CqContext.intrchar; /* restore INTR */
+  term.c_cc[VINTR] = Context.intrchar; /* restore INTR */
   tcsetattr(PollInputfd, TCSANOW, &term);
 #endif
   
@@ -331,7 +334,7 @@ int cdgetx ( char pmt[], int lin, int col, char terms[], char str[],
 /*       maxlen - size of str */
 /*       append_flg - return for more input flag */
 /*       do_append_flg - append operation acceptable flag */
-/*       doecho - whether or not to echo input 
+/*       doecho - whether or not to echo input */
 /* description */
 /*    This routine prompts for input from the terminal. The prompt ``pmt'' */
 /*    and initial contents of ``str'' and displayed and then input is read */
@@ -381,17 +384,19 @@ int cdgetp ( char pmt[], int lin, int col, char terms[], char str[],
       ch = iogchar();
 
       if ( terms[0] != EOS )
-	if ( c_index ( terms, ch ) != ERR )
-	  break;
-	else if ( ch == TERM_NORMAL )
-	  break;
+	{
+	  if ( c_index ( terms, ch ) != ERR )
+	    break;
+	  else if ( ch == TERM_NORMAL )
+	    break;
+	}
 
 				/* translate KP keys (if any)
 				   into 'direction' keys. */
       (void)KP2DirKey(&ch);
 
       if (ch != TERM_NORMAL && ch != TERM_EXTRA && StrInit == TRUE &&
-	  isprint(ch))
+	  isprint(ch & 0xff))
 	{			/* clear out the preload */
 	  str[0] = ch;
 	  str[1] = EOS;
@@ -405,7 +410,7 @@ int cdgetp ( char pmt[], int lin, int col, char terms[], char str[],
 	  continue;
 	}
       else
-	if (!isprint(ch))
+	if (!isprint(ch & 0xff))
 	  {			/* redraw init str in std input color */
 	    attrset(NoColor);
 	    cdclra ( lin, icol, lin, cdcols() );
@@ -483,9 +488,11 @@ int cdgetp ( char pmt[], int lin, int col, char terms[], char str[],
 	      str[0] = EOS;
 	    }
 	}
-      else if ( ch == 0x0c )
-	cdredo();
-      else if ( ! isprint ( ch ) )
+      else if ( ch == TERM_REDRAW )
+	{
+	  cdredo();
+	}
+      else if ( ! isprint ( ch & 0xff) )
 	{
 #ifdef DEBUG_IO
 	  clog("cdgetp1:Got a strange char: ascii %d, errno=%d", ch, errno);
@@ -568,21 +575,16 @@ int cdgetp ( char pmt[], int lin, int col, char terms[], char str[],
 
 void cdbeep(void)
 {
-  int i;
-  static int old = 0;
+  time_t i;
+  static time_t old = 0;
   
-  if (conf_DoLimitBell == TRUE)	/* limit beeps to no more than 1 per sec */
-    {
-      i = time(0);
+  i = time(0);
       
-      if (i != old)
-	{
-	  old = i;
-	  beep();
-	}
+  if (i != old)
+    {
+      old = i;
+      beep();
     }
-  else
-    beep();			/* else beep whenever the urge hits you */
   
   return;
 }
@@ -595,6 +597,7 @@ void cdbeep(void)
 /*    This routine must be called before all others in cdlb. */
 void cdinit(void)
 {
+
 #if defined(HAVE_TERMIOS_H)
   struct termios term;
 #endif
@@ -654,16 +657,26 @@ void cdinit(void)
 				   DEL to be used for deleting text
 				   for example */
 
-  CqContext.intrchar = 0;	/* default - nothing */
+  Context.intrchar = 0;	/* default - nothing */
 
 #if defined (HAVE_TERMIOS_H)
   tcgetattr(PollInputfd, &term);
-  CqContext.intrchar = term.c_cc[VINTR]; /* save it */
+  Context.intrchar = term.c_cc[VINTR]; /* save it */
   term.c_cc[VINTR] = 0x03;	/* ^C - harmless */
   tcsetattr(PollInputfd, TCSANOW, &term);
 #endif
 
   cdclear();
+
+#ifdef CONQGL
+  if (!GLinited)
+    {
+      InitGL(0, NULL);
+      GLinited = TRUE;
+      procEvents();
+    }
+#endif
+
   return;
   
 }
@@ -852,7 +865,7 @@ void cdputs ( char str[], int lin, int col )
   
   strcpy(tmpstr, str);
   
-  len = CqContext.maxcol - col; /* - 1; /* max str that will fit on screen */
+  len = Context.maxcol - col; /* max str that will fit on screen */
   slen = strlen(tmpstr);
   
   len = (len >= 0) ? len : 0;
@@ -875,7 +888,8 @@ void cdputs ( char str[], int lin, int col )
 /*   refresh next time cdrefresh() is called. */
 void cdredo(void)
 {
-  cdclear();
+  wrefresh(curscr);
+
   return;
 }
 
