@@ -4,9 +4,9 @@
  *
  * conf.c - handle configuration issues, both personal and system wide.
  *
- * $Id: conf.c,v 1.9 2001/01/03 20:35:45 jon Exp $
+ * $Id: conf.c,v 1.16 2003/12/21 22:41:01 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /**********************************************************************/
@@ -19,14 +19,28 @@
 
 #include "global.h"
 #include "conqcom.h"
+#include "color.h"
+#include "datatypes.h"
+#include "conqnet.h"
 
-#define NOEXTERN
+#define CONF_NOEXTERN
 #include "conf.h"
-#undef NOEXTERN
+#undef CONF_NOEXTERN
 
-		/* For id purposes... */
-static char *confId = "$Id: conf.c,v 1.9 2001/01/03 20:35:45 jon Exp $";
+#include "protocol.h"
 
+#define HOME_BUFSZ 1024
+
+/* if this is set, then the conquest client is running as the shell on
+   a telnet based server.  In this case, we will never try to load or save
+   the user config file.   GetConf() will only set the defaults. */
+static int telnetClient = FALSE;
+
+void confSetTelnetClientMode(int telnetc)
+{
+  telnetClient = telnetc;
+  return;
+}
 
 /* GetSysConf(int checkonly) - load system-wide configuration values */
 /*  If checkonly is TRUE, then just load internel defaults, and the existing
@@ -37,28 +51,30 @@ static char *confId = "$Id: conf.c,v 1.9 2001/01/03 20:35:45 jon Exp $";
 int GetSysConf(int checkonly)
 {
   FILE *conf_fd;
-  int i, j, n;
-  char conf_name[256];
+  int i, j;
+  char conf_name[MID_BUFFER_SIZE];
   char buffer[BUFFER_SIZE];
   int FoundOne = FALSE;
   int buflen;
-  char *bufptr, *cptr;
+  char *bufptr;
 
 				/* init some defaults */
-  sysconf_DoLRTorpScan = TRUE;
-  sysconf_DoLocalLRScan = TRUE;
-  sysconf_DoETAStats = TRUE;
-  sysconf_AllowFastUpdate = TRUE;
-  sysconf_NoDoomsday = FALSE;
-  sysconf_DoRandomRobotKills = FALSE;
-  sysconf_AllowSigquit = FALSE;
-  sysconf_AllowSwitchteams = TRUE;
-  sysconf_UserExpiredays = DEFAULT_USEREXPIRE;
-  sysconf_LogMessages = FALSE;
-  sysconf_AllowRefits = TRUE;
+  SysConf.NoDoomsday = FALSE;
+  SysConf.DoRandomRobotKills = FALSE;
+  SysConf.AllowVacant = FALSE;
+  SysConf.AllowSwitchteams = TRUE;
+  SysConf.UserExpiredays = DEFAULT_USEREXPIRE;
+  SysConf.LogMessages = FALSE;
+  SysConf.AllowRefits = TRUE;
+  SysConf.AllowSlingShot = FALSE;
+
+  strncpy(SysConf.ServerName, "Generic Conquest Server", 
+	  CONF_SERVER_NAME_SZ);
+  strncpy(SysConf.ServerMotd, "Keep your shields up in battle.",
+	  CONF_SERVER_MOTD_SZ);
 
 				/* start building the filename */
-  sprintf(conf_name, "%s/%s", CONQHOME, SYSCONFIG_FILE);
+  snprintf(conf_name, sizeof(conf_name)-1, "%s/%s", CONQETC, SYSCONFIG_FILE);
 
   if ((conf_fd = fopen(conf_name, "r")) == NULL)
     {
@@ -68,13 +84,13 @@ int GetSysConf(int checkonly)
 
       clog("GetSysConf(): fopen(%s) failed: %s",
 	       conf_name,
-	       sys_errlist[err]);
+	       strerror(err));
 
       if (checkonly != TRUE)
 	{
 	  fprintf(stderr, "Error opening system-wide config file: %s: %s\n",
 		  conf_name,
-		  sys_errlist[err]);
+		  strerror(err));
 	  
 	  fprintf(stderr, "'conqoper -C' needs to be run. Using internal defaults.\n");
 	  sleep(4);
@@ -114,7 +130,7 @@ int GetSysConf(int checkonly)
 
       FoundOne = FALSE;
 
-      for (j = 0; (j < SYSCF_END) && (FoundOne == FALSE); j++)
+      for (j = 0; (j < SysCfEnd) && (FoundOne == FALSE); j++)
 	{
 	  if (SysConfData[j].ConfName != NULL)
 	    if (strncmp(SysConfData[j].ConfName, 
@@ -163,6 +179,18 @@ int GetSysConf(int checkonly)
 			    FoundOne = TRUE;
 			  }
 			break;
+
+		      case CTYPE_STRING:
+			memset((char *)(SysConfData[j].ConfValue), 0,
+			       SysConfData[j].max);
+			strncpy((char *)(SysConfData[j].ConfValue), 
+				bufptr, SysConfData[j].max);
+			((char *)SysConfData[j].ConfValue)[SysConfData[j].max - 1] = 0;
+
+			SysConfData[j].Found = TRUE;
+			FoundOne = TRUE;
+			break;
+
 		      
 		      } /* switch */
 		  } /* if */
@@ -175,7 +203,7 @@ int GetSysConf(int checkonly)
     {
       fprintf(stderr, "GetSysConf(): Error reading %s: %s\n",
 	      conf_name,
-	      sys_errlist[errno]);
+	      strerror(errno));
       fclose(conf_fd);
       
       return(ERR);
@@ -205,7 +233,7 @@ int GetSysConf(int checkonly)
     }
   else
     {				/* version found. check everything else */
-      for (i=0; i<SYSCF_END; i++)
+      for (i=0; i<SysCfEnd; i++)
 	{
 	  if (SysConfData[i].ConfType != CTYPE_NULL)
 	    if (SysConfData[i].Found != TRUE)
@@ -227,38 +255,50 @@ int GetSysConf(int checkonly)
 	}
     }
 
-
-
-
   return(TRUE);
 }
 
 				/* get user's configuration */
-int GetConf(int isremote, int usernum)
+int GetConf(int usernum)
 {
   FILE *conf_fd;
   int i, j, n;
   char conf_name[MID_BUFFER_SIZE];
   char *homevar, *cptr;
+  char home[HOME_BUFSZ];
   char buffer[BUFFER_SIZE];
   int buflen;
   char *bufptr;
   int FoundOne = FALSE;
 
 				/* init some defaults */
-  conf_MessageBell = TRUE;
-  conf_NoColor = FALSE;
-  conf_NoRobotMsgs = FALSE;
-  conf_RecPlayerMsgs = TRUE;
-  conf_DoFastUpdate = TRUE;
-  conf_DoLimitBell = TRUE;
-  conf_ClearOldMsgs = TRUE;
+  UserConf.DoAlarms = TRUE;
+  UserConf.ShowPhasers = TRUE;
+  UserConf.ShowPlanNames = TRUE;
+  UserConf.DoIntrudeAlert = TRUE;
+  UserConf.DoNumMap = TRUE;
+  UserConf.Terse = FALSE;
+  UserConf.DoExplode = TRUE;
+  UserConf.MessageBell = TRUE;
+  UserConf.NoColor = FALSE;
+  UserConf.NoRobotMsgs = FALSE;
+  UserConf.UpdatesPerSecond = 5;	/* default of 5 per sec */
+  UserConf.DistressToFriendly = FALSE;
+  UserConf.AltHUD = FALSE;
+  UserConf.DoLRTorpScan = TRUE;
+  UserConf.DoLocalLRScan = TRUE;
+  UserConf.DoETAStats = TRUE;
 
   for (i=0; i<MAX_MACROS; i++)
     {
-      conf_MacrosF[i][0] = EOS;
+      UserConf.MacrosF[i][0] = EOS;
     }
 
+
+  /* a telnet client leaves here after the defaults are set */
+  if (telnetClient)
+    return TRUE;
+
 				/* start building the filename */
   if ((homevar = getenv("HOME")) == NULL)
     {
@@ -268,26 +308,22 @@ int GetConf(int isremote, int usernum)
       return(ERR);
     }
 
-  if (isremote == TRUE && usernum > 0)
-    {				/* build the remote user version */
-      umask(007);
-      sprintf(conf_name, "%s/%s.%d", homevar, CONFIG_FILE, usernum);
-    }
-  else
-      sprintf(conf_name, "%s/%s", homevar, CONFIG_FILE);
+  memset(home, 0, HOME_BUFSZ);
+  strncpy(home, homevar, HOME_BUFSZ - 1);
 
+  snprintf(conf_name, sizeof(conf_name)-1, "%s/%s", home, CONFIG_FILE);
 
   if ((conf_fd = fopen(conf_name, "r")) == NULL)
     {
       if (errno != ENOENT)
 	{
-	  clog("GetConf(): fopen(%s) failed: %s",
+	  clog("GetConf(): fopen(%s) failed: %s, using defaults",
 	       conf_name,
-	       sys_errlist[errno]);
+	       strerror(errno));
 	  
-	  fprintf(stderr, "Error opening config file: %s: %s\n",
+	  fprintf(stderr, "Error opening config file: %s: %s, using defaults\n",
 		  conf_name,
-		  sys_errlist[errno]);
+		  strerror(errno));
 
 	  return(ERR);
 	}
@@ -332,7 +368,7 @@ int GetConf(int isremote, int usernum)
 
       FoundOne = FALSE;
 
-      for (j = 0; (j < CF_END) && (FoundOne == FALSE); j++)
+      for (j = 0; (j < CfEnd) && (FoundOne == FALSE); j++)
 	{
 	  if (ConfData[j].ConfName != NULL)
 	    if(strncmp(ConfData[j].ConfName, 
@@ -421,7 +457,7 @@ int GetConf(int isremote, int usernum)
     {
       fprintf(stderr, "GetConf(): Error reading %s: %s\n",
 	      conf_name,
-	      sys_errlist[errno]);
+	      strerror(errno));
       fclose(conf_fd);
       
       return(ERR);
@@ -443,7 +479,7 @@ int GetConf(int isremote, int usernum)
     }
   else
     {				/* version found. check everything else */
-      for (i=0; i<CF_END; i++)
+      for (i=0; i<CfEnd; i++)
 	{
 	  if (ConfData[i].ConfType != CTYPE_NULL && 
 	      ConfData[i].ConfType != CTYPE_MACRO )
@@ -466,23 +502,28 @@ int SaveUserConfig(int unum)
 {
   char conf_name[MID_BUFFER_SIZE];
   char *homevar;
+  char home[HOME_BUFSZ];
+
+  /* telnet shell clients can't save their configs :( */
+  if (telnetClient)
+    {
+      InitColors();
+      return TRUE;
+    }
 
 				/* start building the filename */
   if ((homevar = getenv("HOME")) == NULL)
     {
-      clog("SaveAndReloadUserConfig(): getenv(HOME) failed");
+      clog("SaveUserConfig(): getenv(HOME) failed");
 
       fprintf(stderr, "SaveUserConfig(): Can't get HOME environment variable. Exiting\n");
       return(ERR);
     }
 
-  if (unum > 0 && Users[unum].type == UT_REMOTE)
-    {				/* build the remote user version */
-      umask(007);
-      sprintf(conf_name, "%s/%s.%d", homevar, CONFIG_FILE, unum);
-    }
-  else
-      sprintf(conf_name, "%s/%s", homevar, CONFIG_FILE);
+  memset(home, 0, HOME_BUFSZ);
+  strncpy(home, homevar, HOME_BUFSZ - 1);
+
+  snprintf(conf_name, sizeof(conf_name)-1, "%s/%s", home, CONFIG_FILE);
 
 #ifdef DEBUG_OPTIONS
   clog("SaveUserConfig(): saving user config: conf_name = '%s'", conf_name);
@@ -647,23 +688,28 @@ int MakeConf(char *filename)
   FILE *conf_fd;
   int i, j, n;
 
+
+  /* a telnet client should never get here, but... */
+  if (telnetClient)
+    return TRUE;
+
   unlink(filename);
 
   if ((conf_fd = fopen(filename, "w")) == NULL)
     {
       clog("Makeconf(): fopen(%s) failed: %s",
 	   filename,
-	   sys_errlist[errno]);
+	   strerror(errno));
 
       fprintf(stderr, "Error creating %s: %s\n",
 	      filename,
-	      sys_errlist[errno]);
+	      strerror(errno));
 
       return(ERR);
     }
 
 
-  for (j=0; j<CF_END; j++)
+  for (j=0; j<CfEnd; j++)
     {
 				/* option header first */
       i = 0;
@@ -737,7 +783,7 @@ int MakeSysConf()
   char conf_name[BUFFER_SIZE];
   int i, j, n;
 
-  sprintf(conf_name, "%s/%s", CONQHOME, SYSCONFIG_FILE);
+  snprintf(conf_name, sizeof(conf_name)-1, "%s/%s", CONQETC, SYSCONFIG_FILE);
   umask(002);
   unlink(conf_name);
 
@@ -745,17 +791,18 @@ int MakeSysConf()
     {
       clog("MakeSysconf(): fopen(%s) failed: %s",
 	   conf_name,
-	   sys_errlist[errno]);
+	   strerror(errno));
       
       fprintf(stderr, "Error creating %s: %s\n",
 	      conf_name,
-	      sys_errlist[errno]);
+	      strerror(errno));
       return(ERR);
     }
 
+  clog("OPER: Updating %s file...", conf_name);
   fprintf(stderr, "Updating %s file...", conf_name);
 
-  for (j=0; j<SYSCF_END; j++)
+  for (j=0; j<SysCfEnd; j++)
     {
 				/* option header first */
       i = 0;
@@ -778,6 +825,11 @@ int MakeSysConf()
       if (SysConfData[j].ConfType != CTYPE_NULL)
 	switch (SysConfData[j].ConfType)
 	  {
+	  case CTYPE_STRING:
+	    fprintf(sysconf_fd, "%s%s\n",
+		    SysConfData[j].ConfName,
+		    (char *)SysConfData[j].ConfValue);
+	    break;
 	  case CTYPE_MACRO:
 	    for (n=0; n < MAX_MACROS; n++)
 	      {
@@ -819,3 +871,30 @@ int MakeSysConf()
   return(TRUE);
 }
 
+Unsgn32 getServerFlags(void)
+{
+  Unsgn32 f;
+
+  /* get the current flags */
+  f = SPSSTAT_FLAGS_NONE;
+
+  if (SysConf.AllowRefits)
+    f |= SPSSTAT_FLAGS_REFIT;
+
+  if (SysConf.AllowVacant)
+    f |= SPSSTAT_FLAGS_VACANT;
+
+  if (SysConf.AllowSlingShot)
+    f |= SPSSTAT_FLAGS_SLINGSHOT;
+
+  if (SysConf.NoDoomsday)
+    f |= SPSSTAT_FLAGS_NODOOMSDAY;
+
+  if (SysConf.DoRandomRobotKills)
+    f |= SPSSTAT_FLAGS_KILLBOTS;
+
+  if (SysConf.AllowSwitchteams)
+    f |= SPSSTAT_FLAGS_SWITCHTEAM;
+
+  return f;
+}
