@@ -2,9 +2,9 @@
 
 /************************************************************************
  *
- * $Id: conqai.c,v 1.9 2001/01/03 20:35:47 jon Exp $
+ * $Id: conqai.c,v 1.16 2004/01/03 09:05:02 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /*            Copyright (C)1983-1986 by Jef Poskanzer and Craig Leres */
@@ -23,103 +23,17 @@
 /*                                                                    */
 /**********************************************************************/
 
-#ifdef CONQAIMAIN
-#define NOEXTERN
-#endif
-
 #include "conqdef.h"
 #include "conqcom.h"
-#include "conqcom2.h"
+#include "context.h"
 #include "global.h"
+#include "conf.h"
 #include "user.h"
 
 static int nenum; 
 static int debug; 
 static real dne, ane;
 
-/*  conqai - robot AI test program */
-#ifdef CONQAIMAIN
-
-main(int argc, char *argv[])
-{
-  int i;
-  string usage="usage: conqai [-vd]";
-  char arg;
-  
-  /* First things first. */
-  
-  if ((ConquestUID = GetConquestUID()) == ERR)
-    {
-      fprintf(stderr, "conqai: GetConquestUID() failed\n");
-      exit(1);
-    }
-  
-  if ((ConquestGID = GetConquestGID()) == ERR)
-    {
-      fprintf(stderr, "conqai: GetConquestGID() failed\n");
-      exit(1);
-    }
-  
-  if (GetSysConf(FALSE) == ERR)
-    {
-#ifdef DEBUG_CONFIG
-      clog("%s@%d: main(): GetSysConf() returned ERR.", __FILE__, __LINE__);
-#endif
-/*      exit(1);*/
-    }
-  
-  if (setgid(ConquestGID) == -1)
-    {
-      clog("conqai: setgid(%d): %s",
-	   ConquestGID,
-	   sys_errlist[errno]);
-      fprintf(stderr, "conqai: setgid(): failed\n");
-      exit(1);
-    }
-  
-#ifdef USE_SEMS
-  if (GetSem() == ERR)
-    {
-      fprintf(stderr, "GetSem() failed to get semaphores. exiting.\n");
-      exit(1);
-    }
-#endif
-  
-  
-  
-  map_common();
-  
-  if ( *CBlockRevision != COMMONSTAMP )
-    error( "conqai: Common block ident mismatch.  \nInitialize the Universe via conqoper." );
-  
-  
-  debug = FALSE;
-  
-  while ((arg = getopt(argc, argv, "d")) != EOF)
-    {
-      switch (arg)
-	{
-	case 'r':
-	  exitai();		/* release robot control to driver */
-	  printf("The conquest driver now has control of the robots\n");
-	  exit(0);
-	  break;		/* NOTREACHED */
-	case 'd':
-	  debug = TRUE;
-	  break;
-	default:
-	  printf("Options: -d = debugging, -r = return robot control to driver\n");
-	  exit(1);
-	}
-    }
-  robotloop();
-  
-  exit(0);
-  
-}
-
-#endif /* CONQAIMAIN */
-
 /*  buildai - construct the robot data base */
 /*  SYNOPSIS */
 /*    int snum, vars(MAX_VAR), nenum */
@@ -167,7 +81,7 @@ void buildai( int snum, int vars[], int *bnenum, real *bdne, real *bane )
   *bdne = 1e9;
   if ( findspecial( snum, SPECIAL_ENEMYSHIP, 0, bnenum, &xnenum ) )
     {
-      if ( Ships[*bnenum].cloaked )
+      if ( SCLOAKED(*bnenum) )
 	{
 	  x = rndnor( Ships[*bnenum].x, CLOAK_SMEAR_DIST );
 	  y = rndnor( Ships[*bnenum].y, CLOAK_SMEAR_DIST );
@@ -183,7 +97,7 @@ void buildai( int snum, int vars[], int *bnenum, real *bdne, real *bane )
       /* Enemy is cloaked (-) */
       if ( *bdne < ACCINFO_DIST )
 	{
-	  AIBOOLEAN( vars[VAR_ENEMYCLOAKED], Ships[*bnenum].cloaked == TRUE );
+	  AIBOOLEAN( vars[VAR_ENEMYCLOAKED], SCLOAKED(*bnenum));
 	  AISCALE( vars[VAR_ENEMYDAMAGE], Ships[*bnenum].damage, 10.0 );
 	} 
     }
@@ -193,7 +107,7 @@ void buildai( int snum, int vars[], int *bnenum, real *bdne, real *bane )
   AISCALE( vars[VAR_DAMAGE], Ships[snum].damage, 10.0 );
   
   /* Possible ship damage from enemy torps (10) */
-  if ( Ships[snum].talert )
+  if ( STALERT(snum) )
     {
       dam = 0.0;
       for ( i = 1; i <= MAXSHIPS; i = i + 1 )
@@ -240,13 +154,13 @@ void buildai( int snum, int vars[], int *bnenum, real *bdne, real *bane )
   AISCALE( vars[VAR_WARP], Ships[snum].dwarp, 1.0 );
   
   /* Ship shields are up (-) */
-  AIBOOLEAN( vars[VAR_SHUP], Ships[snum].shup );
+  AIBOOLEAN( vars[VAR_SHUP], SSHUP(snum) );
   
   /* Are in repair mode (-) */
-  AIBOOLEAN( vars[VAR_REPAIRING], Ships[snum].rmode );
+  AIBOOLEAN( vars[VAR_REPAIRING], SREPAIR(snum) );
   
   /* Are cloaked (-) */
-  AIBOOLEAN( vars[VAR_CLOAKED], Ships[snum].cloaked );
+  AIBOOLEAN( vars[VAR_CLOAKED], SCLOAKED(snum) );
   
   /* Weapons are allocated (-) */
   AIBOOLEAN( vars[VAR_WALLOC], Ships[snum].weapalloc > 50 );
@@ -284,9 +198,12 @@ void defend( int attacker, int pnum )
   
   /* See if there are any team ships to defend. */
   for ( i = 1; i <= MAXSHIPS; i = i + 1 )
-    if ( Ships[i].status == SS_LIVE )
-      if ( Ships[i].team == team )
-	return;
+    if ( Ships[i].status == SS_LIVE ) /* live */
+      if ( Ships[i].team == team ) /* same team */
+	if (Users[Ships[i].unum].robot || !SVACANT(i)) 
+	  {   /* robot or non-vacant human */
+	    return;
+	  }
   
   /* Count how many robot users are on the right team and can play. */
   j = 0;
@@ -378,7 +295,7 @@ void executeai( int snum, int token )
 		 Ships[snum].dhead = Ships[snum].head; \
 	}                      \
 	if ( (x) > 0.0 )             \
-		 Ships[snum].rmode = FALSE;  \
+		 SFCLR(snum, SHIP_F_REPAIR);  \
 	Ships[snum].dwarp = (x);          \
   }
     
@@ -448,7 +365,10 @@ void executeai( int snum, int token )
       launch( snum, ane, 3, LAUNCH_NORMAL );
       break;
     case ROB_SHIELD:
-      Ships[snum].shup = (Ships[snum].shup) ? FALSE : TRUE;
+      if (SSHUP(snum))
+	SFCLR(snum, SHIP_F_SHUP);
+      else
+	SFSET(snum, SHIP_F_SHUP);
       break;
     case ROB_WARP_0:
       SETWARP( 0.0 );
@@ -469,13 +389,16 @@ void executeai( int snum, int token )
       SETCOURSE( mod360( ane + 180.0 + rnduni( -10.0, 10.0 ) ) );
       break;
     case ROB_SILENT:
-      if ( ! Ships[snum].cloaked )
+      if ( ! SCLOAKED(snum) )
 	Ships[snum].dwarp = 0.0;
-      Ships[snum].cloaked = ! Ships[snum].cloaked;
+      if (SCLOAKED(snum))
+	SFCLR(snum, SHIP_F_CLOAKED);
+      else
+	SFSET(snum, SHIP_F_CLOAKED);
       break;
     case ROB_INSULT:
       robreply( buf );
-      stormsg( snum, nenum, buf );
+      stormsgf( snum, nenum, buf, MSG_FLAGS_ROBOT );
       break;
     case ROB_READMSG:
       /* Try to read a message and reply to it */
@@ -490,26 +413,26 @@ void executeai( int snum, int token )
 		continue; 	/* don't talk back to planets */
 
 	      if ( j > 0 && j <= MAXSHIPS )
-		if ( Ships[j].robot )
+		if ( SROBOT(j) )
 		  continue; 	/* don't talk back to robots */
 
 	      if (j == MSG_GOD)
 		continue;	/* don't talk back to GOD */
 
-	      if (j == MSG_COMP)
+	      if (j == MSG_COMP || (Msgs[i].flags & MSG_FLAGS_TERSABLE))
 		continue;	/* don't talk back to the computer */
    
 	      robreply( buf );
-	      stormsg( snum, j, buf );
+	      stormsgf( snum, j, buf, MSG_FLAGS_ROBOT );
 	      break;
 	    }
 	}
       break;
     case ROB_MESSAGE:
-      stormsg( snum, MSG_ALL, "Give me drugs." );
+      stormsgf( snum, MSG_ALL, "Give me drugs.", MSG_FLAGS_ROBOT );
       break;
     case ROB_TAKEDRUGS:
-      stormsg( snum, MSG_ALL, "I'm on drugs." );
+      stormsgf( snum, MSG_ALL, "I'm on drugs.", MSG_FLAGS_ROBOT );
       break;
     case ROB_DETONATE:
       enemydet( snum );
@@ -528,7 +451,7 @@ void executeai( int snum, int token )
 	  }
       break;
     case ROB_REPAIR:
-      Ships[snum].rmode = TRUE;
+      SFSET(snum, SHIP_F_REPAIR);
       break;
     default:
       robstr( token, buf );
@@ -606,12 +529,12 @@ int newrob( int *snum, int unum )
   /* Initialize the ship. */
   PVLOCK(&ConqInfo->lockword);
   initship( *snum, unum );
-  Ships[*snum].robot = TRUE;			/* we're a robot */
+  SFSET(*snum, SHIP_F_ROBOT);			/* we're a robot */
 
 				/* see if we should randomize it's strength
 				   otherwise do nothing since sstrkills
 				   was initialized to 0.0 in initship */
-  if (sysconf_DoRandomRobotKills == TRUE)
+  if (SysConf.DoRandomRobotKills == TRUE)
     {
 				/* randomize the robot's 'strength' */
       Ships[*snum].strkills = 
@@ -623,7 +546,7 @@ int newrob( int *snum, int unum )
   Ships[*snum].unum = unum;
   Ships[*snum].team = Users[unum].team;
 
-  if (sysconf_AllowRefits)
+  if (SysConf.AllowRefits)
     Ships[*snum].shiptype = rndint( 0, MAXNUMSHIPTYPES - 1 );
   else
     Ships[*snum].shiptype = Teams[Ships[*snum].team].shiptype;
@@ -637,22 +560,13 @@ int newrob( int *snum, int unum )
   Ships[*snum].weapalloc = 30;
   Ships[*snum].engalloc = 100 - Ships[*snum].weapalloc;
 
-  
-  for ( i = 0; i < MAXOPTIONS; i = i + 1 )
-    Ships[*snum].options[i] = Users[unum].options[i];
-  Ships[*snum].options[OPT_INTRUDERALERT] = TRUE;	/* want intruder alerts */
-  Ships[*snum].options[OPT_TERSE] = TRUE;		/* don't want stupid
-						   messages */
-  Ships[*snum].options[OPT_ALARMBELL] = FALSE; /* don't want beeping during a
-					    watch */
-
   for ( i = 0; i < NUMPLAYERTEAMS; i = i + 1 )
     {
       /* Robots are peace (and fun) loving. */
       Ships[*snum].rwar[i] = FALSE;
       Ships[*snum].war[i] = FALSE;
     }
-  stcpn ( Users[unum].alias, Ships[*snum].alias, MAXUSERPNAME );	/* -[] -[] */
+  stcpn ( Users[unum].alias, Ships[*snum].alias, MAXUSERPNAME );	
   
   /* Place the ship. */
   if ( Planets[Teams[Ships[*snum].team].homeplanet].primary == 
@@ -756,7 +670,7 @@ void robotloop(void)
     {
       for ( s = 1; s <= MAXSHIPS; s = s + 1 )
 	if ( Ships[s].status == SS_LIVE )
-	  if ( Ships[s].robot )
+	  if ( SROBOT(s) )
 	    {
 	      /* This code taken from conqdriv. */
 	      initstats( &Ships[s].ctime, &j );
@@ -780,7 +694,7 @@ void robotloop(void)
 /*    robreply( buf ) */
 void robreply( char buf[] )
 {
-  const int NUMRREPLIES = 59;
+  const int NUMRREPLIES = 60;
   static char *robreplies[] = {
     "Hey sucker, eat me!",
     "Take off, eh?",
@@ -804,6 +718,7 @@ void robreply( char buf[] )
     "Do not worry about birth control devices, I have many.",
     "I bet you only talk big.",
     "Kiss my ram memory.",
+    "Kiss my shiny metal ass.",
     "Do you think we can use battery operated devices under water?",
     "Nothing shocks me - I'm a robot.",
     "Ok, eh?",
@@ -964,7 +879,7 @@ void robstr( int token, char buf[] )
 /*    token = tableai( vars ) */
 int tableai( int vars[] )
 {
-  int status, token, rule, i;
+  int status, token = ERR, rule, i;
   int rbits;
   
   /* Set all bits. */
