@@ -2,9 +2,9 @@
 
 /************************************************************************
  *
- * $Id: conqoper.c,v 1.14 2001/12/26 22:06:12 jon Exp $
+ * $Id: conqoper.c,v 1.21 2004/01/08 06:12:28 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /*                               C O N Q O P E R */
@@ -25,13 +25,18 @@
 /**********************************************************************/
 
 #define NOEXTERN
+#include "global.h"
+#include "conf.h"
 #include "conqdef.h"
 #include "conqcom.h"
-#include "conqcom2.h"
+#include "context.h"
 #include "global.h"
 #include "color.h"
+#include "record.h"
+#include "display.h"
+#include "clientlb.h"
+#include "clntauth.h"
 
-static char *conqoperId = "$Id: conqoper.c,v 1.14 2001/12/26 22:06:12 jon Exp $";
 static char cbuf[MID_BUFFER_SIZE]; /* general purpose buffer */
 
 				/* option masks */
@@ -43,8 +48,14 @@ static char cbuf[MID_BUFFER_SIZE]; /* general purpose buffer */
 
 static char operName[MAXUSERNAME];
 
+void DoConqoperSig(int sig);
+void astoperservice(int sig);
+void EnableConqoperSignalHandler(void);
+void operStopTimer(void);
+void operSetTimer(void);
+
 /*  conqoper - main program */
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
   int i;
   char msgbuf[128];
@@ -57,7 +68,8 @@ main(int argc, char *argv[])
 
   OptionAction = OP_NONE;
 
-  glname( operName, MAXUSERNAME );
+  strncpy(operName, glname(), MAXUSERNAME);
+  operName[MAXUSERNAME - 1] = 0;
 
   if ( ! isagod(-1) )
     {
@@ -106,9 +118,9 @@ main(int argc, char *argv[])
 	break;
       }
 
-  if ((ConquestUID = GetConquestUID()) == ERR)
+  if ((ConquestUID = GetUID(ROOT_USER)) == ERR)
     {
-      fprintf(stderr, "conqoper: GetConquestUID() failed\n");
+      fprintf(stderr, "conqoper: GetUID() failed\n");
       exit(1);
     }
   
@@ -124,20 +136,15 @@ main(int argc, char *argv[])
       /* need to be conq grp for these */
       if (setgid(ConquestGID) == -1)
 	{
-	  clog("conqoper: setgid(%d): %s",
-	       ConquestGID,
-	       sys_errlist[errno]);
 	  fprintf(stderr, "conqoper: setgid(): failed\n");
 	  exit(1);
 	}
 
-#ifdef USE_SEMS
       if (GetSem() == ERR)
 	{
 	  fprintf(stderr, "GetSem() failed to get semaphores. exiting.\n");
 	  exit(1);
 	}
-#endif
 
       GetSysConf(TRUE);		/* init defaults... */
       map_common();		/* Map the conquest universe common block */
@@ -195,7 +202,9 @@ main(int argc, char *argv[])
       ;
     }
 
-  if (GetConf(FALSE, 0) == ERR)	/* use one if there, else defaults
+  Context.updsec = 2;		/* default upd per sec */
+
+  if (GetConf(0) == ERR)	/* use one if there, else defaults
 				   A missing or out-of-date conquestrc file
 				   will be ignored */
     {
@@ -210,18 +219,16 @@ main(int argc, char *argv[])
     {
       clog("conqoper: setgid(%d): %s",
 	   ConquestGID,
-	   sys_errlist[errno]);
+	   strerror(errno));
       fprintf(stderr, "conqoper: setgid(): failed\n");
       exit(1);
     }
   
-#ifdef USE_SEMS
   if (GetSem() == ERR)
     {
       fprintf(stderr, "GetSem() failed to get semaphores. exiting.\n");
       exit(1);
     }
-#endif
   
 #ifdef SET_PRIORITY
   /* Increase our priority a bit */
@@ -230,7 +237,7 @@ main(int argc, char *argv[])
     {
       clog("conqoper: main(): nice(CONQUEST_PRI (%d)): failed: %s",
 	   CONQUEST_PRI,
-	   sys_errlist[errno]);
+	   strerror(errno));
     }
 #if defined(DEBUG_FLOW)
   else
@@ -245,18 +252,22 @@ main(int argc, char *argv[])
   rndini( 0, 0 );		/* initialize random numbers */
   cdinit();			/* initialize display environment */
   
-  CqContext.unum = MSG_GOD;	/* stow user number */
-  CqContext.snum = ERR;		/* don't display in cdgetp - JET */
-  CqContext.entship = FALSE;	/* never entered a ship */
-  CqContext.histslot = ERR;	/* useless as an op */
-  CqContext.display = TRUE;
-  CqContext.maxlin = cdlins();	/* number of lines */
+  Context.unum = MSG_GOD;	/* stow user number */
+  Context.snum = ERR;		/* don't display in cdgetp - JET */
+  Context.entship = FALSE;	/* never entered a ship */
+  Context.histslot = ERR;	/* useless as an op */
+  Context.lasttang = Context.lasttdist = 0;
+  Context.lasttarg[0] = EOS;
+  Context.display = TRUE;
+  Context.maxlin = cdlins();	/* number of lines */
   
-  CqContext.maxcol = cdcols();	/* number of columns */
+  Context.maxcol = cdcols();	/* number of columns */
+  Context.recmode = RECMODE_OFF;
 
   sprintf(msgbuf, "OPER: User %s has entered conqoper.",
           operName);
   clog(msgbuf);			/* log it too... */
+  stormsg( MSG_COMP, MSG_GOD, msgbuf );
 
   operate();
   
@@ -279,14 +290,16 @@ void bigbang(void)
   cnt = 0;
   for ( snum = 1; snum <= MAXSHIPS; snum = snum + 1 )
     if ( Ships[snum].status == SS_LIVE )
-      for ( i = 0; i < MAXTORPS; i = i + 1 )
-	if ( ! launch( snum, dir, 1, LAUNCH_NORMAL ) )
-	  break;
-	else
-	  {
-	    dir = mod360( dir + 40.0 );
-	    cnt = cnt + 1;
-	  }
+      {
+	for ( i = 0; i < MAXTORPS; i = i + 1 )
+	  if ( ! launch( snum, dir, 1, LAUNCH_NORMAL ) )
+	    break;
+	  else
+	    {
+	      dir = mod360( dir + 40.0 );
+	      cnt = cnt + 1;
+	    }
+      }
   cprintf(MSG_LIN1,0,ALIGN_CENTER, 
   "#%d#bigbang: Fired #%d#%d #%d#torpedos, hoo hah won't they be surprised!", 
 	InfoColor,SpecialColor,cnt,InfoColor );
@@ -322,7 +335,7 @@ void debugdisplay( int snum )
   int i, j, unum, lin, tcol, dcol;
   real x;
   char buf[MSGMAXLINE];
-  char *torpstr;
+  char *torpstr = "???";
   
 #define TOFF "OFF"
 #define TLAUNCHING "LAUNCHING"
@@ -340,7 +353,7 @@ void debugdisplay( int snum )
   cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "    ship:");
   buf[0] = EOS;
   appship( snum, buf );
-  if ( Ships[snum].robot )
+  if ( SROBOT(snum) )
     appstr( " (ROBOT)", buf );
   cprintf( lin, dcol,ALIGN_NONE,"#%d#%s",InfoColor,buf );
   lin++;
@@ -439,7 +452,7 @@ void debugdisplay( int snum )
     {
       sprintf( buf, "%d ", i );
     }
-  if ( Ships[snum].cloaked )
+  if ( SCLOAKED(snum) )
     appstr( "(CLOAKED)", buf );
   cprintf(lin,dcol,ALIGN_NONE,"#%d#%s",LabelColor, buf);
   lin++;
@@ -476,14 +489,14 @@ void debugdisplay( int snum )
   lin++;
   cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "   shields:");
   cprintf(lin,dcol,ALIGN_NONE,"#%d#%0d",InfoColor, round(Ships[snum].shields));
-  if ( ! Ships[snum].shup )
+  if ( ! SSHUP(snum) )
   	cprintf(lin,dcol+5,ALIGN_NONE,"#%d#%c",InfoColor, 'D');
   else
   	cprintf(lin,dcol+5,ALIGN_NONE,"#%d#%c",InfoColor, 'U');
   lin++;
   cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "   sdamage:");
   cprintf(lin,dcol,ALIGN_NONE,"#%d#%0d",InfoColor, round(Ships[snum].damage));
-  if ( Ships[snum].rmode )
+  if ( SREPAIR(snum) )
   	cprintf(lin,dcol+5,ALIGN_NONE,"#%d#%c",InfoColor, 'R');
   lin++;
   cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "  stowedby:");
@@ -528,26 +541,13 @@ void debugdisplay( int snum )
   buf[NUMPLAYERTEAMS+2] = EOS;
   cprintf(lin,dcol,ALIGN_NONE,"#%d#%s",InfoColor, buf);
   
+  lin ++;
+
+  cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "     flags:");
+  cprintf(lin,dcol,ALIGN_NONE,"#%d#0x%04x",InfoColor, Ships[snum].flags);
+
   lin++;
-  cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "   soption:");
-  c_strcpy( "(gpainte)", buf );
-  for ( i = 0; i < MAXOPTIONS; i = i + 1 )
-    if ( Ships[snum].options[i] )
-      buf[i+1] = (char)toupper(buf[i+1]);
-  cprintf(lin,dcol,ALIGN_NONE,"#%d#%s",InfoColor, buf);
-  
-  lin++;
-  cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "   options:");
-  if ( unum >= 0 && unum < MAXUSERS )
-    {
-      c_strcpy( "(gpainte)", buf );
-      for ( i = 0; i < MAXOPTIONS; i = i + 1 )
-	if ( Users[unum].options[i] )
-	  buf[i + 1] = (char)toupper(buf[i + 1]);
-      cprintf(lin,dcol,ALIGN_NONE,"#%d#%s",InfoColor, buf);
-    }
-  
-  lin++;
+
   cprintf(lin,tcol,ALIGN_NONE,"#%d#%s",LabelColor, "   saction:");
   i = Ships[snum].action;
   if ( i != 0 )
@@ -556,7 +556,7 @@ void debugdisplay( int snum )
 	  cprintf(lin,dcol,ALIGN_NONE,"#%d#%s",InfoColor, buf);
     }
   
-  lin = (CqContext.maxlin - (CqContext.maxlin - MSG_LIN1)) - (MAXTORPS+1);  /* dwp */
+  lin = (Context.maxlin - (Context.maxlin - MSG_LIN1)) - (MAXTORPS+1);  /* dwp */
   cprintf(lin,3,ALIGN_NONE,"#%d#%s",LabelColor,
 	 "tstatus    tfuse    tmult       tx       ty      tdx      tdy     twar");
   for ( i = 0; i < MAXTORPS; i = i + 1 )
@@ -580,7 +580,7 @@ void debugdisplay( int snum )
 	  torpstr = TFIREBALL;
 	  break;
 	}
-	  cprintf(lin,3,ALIGN_NONE,"#%d#%s",InfoColor, torpstr);
+      cprintf(lin,3,ALIGN_NONE,"#%d#%s",InfoColor, torpstr);
       if ( Ships[snum].torps[i].status != TS_OFF )
 	{
 	  cprintf(lin,13,ALIGN_NONE,"#%d#%6d",InfoColor, Ships[snum].torps[i].fuse);
@@ -618,7 +618,7 @@ void debugplan(void)
   int i, j, k, cmd, lin, col, olin;
   int outattr;
   static int sv[NUMPLANETS + 1];
-  char buf[MSGMAXLINE], junk[10], uninhab[20];
+  char junk[10], uninhab[20];
   char hd0[MSGMAXLINE*4];
   char *hd1="D E B U G G I N G  P L A N E T   L I S T";
   string hd2="planet        C T arm uih scan        planet        C T arm uih scan";
@@ -756,7 +756,7 @@ void debugplan(void)
 
           cdrefresh();
 	  
-          if (iogtimed( &cmd, 1 ))
+          if (iogtimed( &cmd, 1.0 ))
             {                   /* got a char */
               if (cmd != ' ')
                 {               /* quit */
@@ -1184,31 +1184,8 @@ void operate(void)
 		  xbuf,LabelColor,", eater ",InfoColor,buf);
 	  
 	  /* Line 2. */
-#ifdef USE_SEMS
 	  strcpy(buf, GetSemVal(0));
-#else
-	  c_strcpy( "lockword", buf );
-	  if ( ConqInfo->lockword == 0 )
-	    cntlockword = 0;
-	  else if ( cntlockword < 5 )
-	    cntlockword = cntlockword + 1;
-	  else
-	    upper( buf );
-	  c_strcpy( buf, junk );
-	  appstr( " = %d, ", junk );
-      
-	  c_strcpy( "lockmesg", buf );
-	  if ( ConqInfo->lockmesg == 0 )
-	    cntlockmesg = 0;
-	  else if ( cntlockmesg < 5 )
-	    cntlockmesg = cntlockmesg + 1;
-	  else
-	    upper( buf );
-	  appstr( buf, junk );
-	  appstr( " = %d", junk );
-	  sprintf( buf, junk, ConqInfo->lockword, ConqInfo->lockmesg );
-#endif
-      
+
 	  lin++;
 	  cdclrl( lin, 1 );
 	  attrset(SpecialColor);
@@ -1250,7 +1227,7 @@ void operate(void)
 
 
       /* Get a char with timeout. */
-      if ( ! iogtimed( &ch, 1 ) )
+      if ( ! iogtimed( &ch, 1.0 ) )
 	continue;		/* next */
       switch ( ch )
 	{
@@ -1325,7 +1302,7 @@ void operate(void)
 	  review( MSG_GOD, ConqInfo->glastmsg );
 	  break;
 	case 'm':
-	  sendmsg( MSG_GOD, TRUE );
+	  clntSendMsg( MSG_GOD, TRUE, FALSE );
 	  break;
 	case 'O':
 	  SysOptsMenu();
@@ -1364,7 +1341,7 @@ void operate(void)
 	  break;
 	case 'w':
 	  watch();
-	  stoptimer();		/* to be sure */
+	  operStopTimer();		/* to be sure */
 	  redraw = TRUE;
 	  break;
 	case '/':
@@ -1850,7 +1827,7 @@ void oppedit(void)
       cdmove( 0, 0 );
       cdrefresh();
       
-      if ( ! iogtimed( &ch, 1 ) )
+      if ( ! iogtimed( &ch, 1.0 ) )
 	continue;		/* next */
       switch ( ch )
 	{
@@ -2351,7 +2328,7 @@ void opstats(void)
       attrset(0);
       cdrefresh();
     }
-  while ( !iogtimed( &ch, 1 ) ); /* until */
+  while ( !iogtimed( &ch, 1.0 ) ); /* until */
   
   return;
   
@@ -2373,7 +2350,7 @@ void opteamlist(void)
       putpmt( MTXT_DONE, MSG_LIN2 );
       cdrefresh();
     }
-  while ( !iogtimed( &ch, 1 ) ); /* until */
+  while ( !iogtimed( &ch, 1.0 ) ); /* until */
   
 }
 
@@ -2461,20 +2438,14 @@ void opuadd(void)
     }
   else		
     {
-      if (nameptr != name)	/* a remote user (@) */
-	Users[unum].type = UT_REMOTE;
-      else
-	Users[unum].type = UT_LOCAL;
-
       clog("OPER: %s added user '%s'.",
 	   operName, name);
 
     }
   cdclrl( MSG_LIN1, 2 );
   
-				/* if a remote user, get a pw */
-  if (Users[unum].type == UT_REMOTE)
-    ChangePassword(unum, TRUE);
+
+  ChangePassword(unum, TRUE);
 
   return;
 
@@ -2493,7 +2464,6 @@ void opuedit(void)
   char buf[MSGMAXLINE];
   int ch, left = TRUE;
   char datestr[DATESIZE];
-  static char *prompt = "Use arrow keys to position, [SPACE] to modify";
   static char *prompt2 = "any other key to quit.";
   static char *rprompt = "Use arrow keys to position, [SPACE] to modify, [TAB] to change password";
   char *promptptr;
@@ -2540,30 +2510,13 @@ void opuedit(void)
 	      Users[unum].multiple);
       
       lin++;
-      for ( i = 0; i < MAXOPTIONS; i++ )
+      for ( i = 0; i < MAXOOPTIONS; i++ )
 	{
-      cprintf(lin+i,tcol,ALIGN_NONE,"#%d#%17d:", LabelColor,i);
-	  if ( Users[unum].options[i] )
-      	cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", GreenLevelColor,'T');
-	  else
-      	cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", RedLevelColor,'F');
+	  cprintf(lin+i,tcol,ALIGN_NONE,"#%d#%17d:", LabelColor,i);
+	  cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", RedLevelColor,'F');
 	}
-      cprintf(lin+OPT_PHASERGRAPHICS,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"  Phaser graphics:");
-      cprintf(lin+OPT_PLANETNAMES,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"     Planet names:");
-      cprintf(lin+OPT_ALARMBELL,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"       Alarm bell:");
-      cprintf(lin+OPT_INTRUDERALERT,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"  Intruder alerts:");
-      cprintf(lin+OPT_NUMERICMAP,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"      Numeric map:");
-      cprintf(lin+OPT_TERSE,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"            Terse:");
-      cprintf(lin+OPT_EXPLOSIONS,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"       Explosions:");
-      
-      lin+=(MAXOPTIONS + 1);
+
+      lin+=(MAXOOPTIONS + 1);
       cprintf(lin,tcol,ALIGN_NONE,"#%d#%s", LabelColor,"          Urating:");
       cprintf(lin,dcol,ALIGN_NONE,"#%d#%0g",InfoColor, 
 	      oneplace(Users[unum].rating));
@@ -2593,11 +2546,7 @@ void opuedit(void)
       dcol = 22;
       lcol = dcol - 1;
       
-      if (Users[unum].type == UT_REMOTE)
-	cprintf(lin,tcol,ALIGN_NONE,"#%d#%s#%d#%s", RedLevelColor, 
-		"    (REMOTE) ", LabelColor,"Name:");
-      else
-	cprintf(lin,tcol,ALIGN_NONE,"#%d#%s", LabelColor,"             Name:");
+      cprintf(lin,tcol,ALIGN_NONE,"#%d#%s", LabelColor,"             Name:");
 
 
       cprintf(lin,dcol,ALIGN_NONE,"#%d#%s",InfoColor, 
@@ -2614,11 +2563,11 @@ void opuedit(void)
       lin++;
       for ( i = 0; i < MAXOOPTIONS; i++ )
 	{
-      cprintf(lin+i,tcol,ALIGN_NONE,"#%d#%17d:", LabelColor,i);
+	  cprintf(lin+i,tcol,ALIGN_NONE,"#%d#%17d:", LabelColor,i);
 	  if ( Users[unum].ooptions[i] )
-      	cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", GreenLevelColor,'T');
+	    cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", GreenLevelColor,'T');
 	  else
-      	cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", RedLevelColor,'F');
+	    cprintf(lin+i,dcol,ALIGN_NONE,"#%d#%c", RedLevelColor,'F');
 	}
       cprintf(lin+OOPT_MULTIPLE,tcol,ALIGN_NONE,"#%d#%s", 
 		LabelColor,"         Multiple:");
@@ -2628,8 +2577,8 @@ void opuedit(void)
 		LabelColor," Play when closed:");
       cprintf(lin+OOPT_SHITLIST,tcol,ALIGN_NONE,"#%d#%s", 
 		LabelColor,"          Disable:");
-      cprintf(lin+OOPT_GODMSG,tcol,ALIGN_NONE,"#%d#%s", 
-		LabelColor,"     GOD messages:");
+      cprintf(lin+OOPT_OPER,tcol,ALIGN_NONE,"#%d#%s", 
+		LabelColor,"Conquest Operator:");
       cprintf(lin+OOPT_LOSE,tcol,ALIGN_NONE,"#%d#%s", 
 		LabelColor,"             Lose:");
       cprintf(lin+OOPT_AUTOPILOT,tcol,ALIGN_NONE,"#%d#%s", 
@@ -2732,11 +2681,7 @@ void opuedit(void)
       cprintf(lin,dcol,ALIGN_NONE,"#%d#%0d",InfoColor, 
       	Users[unum].stats[USTAT_ENTRIES]);
       
-      /* Display the stuff */
-      if (Users[unum].type == UT_REMOTE)
-	promptptr = rprompt;
-      else
-	promptptr = prompt;
+      promptptr = rprompt;
 
       cprintf(MSG_LIN1,0,ALIGN_CENTER,"#%d#%s", InfoColor,
 	      promptptr);
@@ -2753,7 +2698,7 @@ void opuedit(void)
       cdrefresh();
       
       /* Now, get a char and process it. */
-      if ( ! iogtimed( &ch, 1 ) )
+      if ( ! iogtimed( &ch, 1.1 ) )
 	continue;		/* next */
       switch ( ch )
 	{
@@ -2888,16 +2833,12 @@ void opuedit(void)
 		else
 		  cdbeep();
 	      else
-		if ( i >= 0 && i < MAXOPTIONS )
-		  Users[unum].options[i] = ! Users[unum].options[i];
-		else
-		  cdbeep();
+		cdbeep();
 	    }
 	  break;
 
 	case TERM_EXTRA:	/* change passwd */
-	  if (Users[unum].type == UT_REMOTE)
-	    ChangePassword(unum, TRUE);
+	  ChangePassword(unum, TRUE);
 	  break;
 	  
 	case 0x0c:
@@ -2937,20 +2878,20 @@ void watch(void)
   else
     {
 	  old_snum = tmp_snum = snum;
-	  CqContext.redraw = TRUE;
+	  Context.redraw = TRUE;
 	  cdclear();
 	  cdredo();
 	  grand( &msgrand );
 
-	  CqContext.snum = snum;		/* so display knows what to display */
-	  setopertimer();
+	  Context.snum = snum;		/* so display knows what to display */
+	  operSetTimer();
 
 	  while (TRUE)	/* repeat */
 	    {
 	      if (!normal)
-		CqContext.display = FALSE; /* can't use it to display debugging */
+		Context.display = FALSE; /* can't use it to display debugging */
 	      else
-		CqContext.display = TRUE;
+		Context.display = TRUE;
 
 		/* set up toggle line display */
 		/* cdclrl( MSG_LIN1, 1 ); */
@@ -2990,7 +2931,7 @@ void watch(void)
 		  ConqInfo->glastmsg = modp1( ConqInfo->glastmsg - 1, MAXMESSAGES );
 	      
 	      /* Get a char with timeout. */
-	      if ( ! iogtimed( &ch, 1 ) )
+	      if ( ! iogtimed( &ch, 1.0 ) )
 		continue; /* next */
 	      cdclrl( MSG_LIN1, 2 );
 	      switch ( ch )
@@ -3007,19 +2948,19 @@ void watch(void)
 		    cdbeep();
 		  break;
 		case 'h':
-		  stoptimer();
+		  operStopTimer();
 		  dowatchhelp();
-		  CqContext.redraw = TRUE;
-		  setopertimer();
+		  Context.redraw = TRUE;
+		  operSetTimer();
 		  break;
 		case 'i':
 		  opinfo( MSG_GOD );
 		  break;
 		case 'k':
-		  kiss(CqContext.snum, TRUE);
+		  kiss(Context.snum, TRUE);
 		  break;
 		case 'm':
-		  sendmsg( MSG_GOD, TRUE );
+		  clntSendMsg( MSG_GOD, TRUE, FALSE );
 		  break;
 		case 'r':  /* just for fun - dwp */
 		  oprobot();
@@ -3028,14 +2969,14 @@ void watch(void)
 		  review( MSG_GOD, ConqInfo->glastmsg );
 		  break;
 		case 0x0c:
-		  stoptimer();
+		  operStopTimer();
 		  cdredo();
-		  CqContext.redraw = TRUE;
-		  setopertimer();
+		  Context.redraw = TRUE;
+		  operSetTimer();
 		  break;
 		case 'q':
 		case 'Q':
-		  stoptimer();
+		  operStopTimer();
 		  return;
 		  break;
 		case 'w': /* look at any ship (live or not) if specifically asked for */
@@ -3046,14 +2987,14 @@ void watch(void)
 			{
 			  old_snum = tmp_snum;
 			  tmp_snum = snum;
-			  CqContext.redraw = TRUE;
+			  Context.redraw = TRUE;
 			}
-		      CqContext.snum = snum;
+		      Context.snum = snum;
 		      if (normal)
 			{
-			  stoptimer();
-			  display( CqContext.snum, headerflag );
-			  setopertimer();
+			  operStopTimer();
+			  display( Context.snum, headerflag );
+			  operSetTimer();
 			}
 		    }
 		  break;
@@ -3072,13 +3013,13 @@ void watch(void)
 			  snum = old_snum;
 			  old_snum = tmp_snum;
 			  
-			  CqContext.snum = snum;
-			  CqContext.redraw = TRUE;
+			  Context.snum = snum;
+			  Context.redraw = TRUE;
 			  if (normal)
 			    {
-			      stoptimer();
-			      display( CqContext.snum, headerflag );
-			      setopertimer();
+			      operStopTimer();
+			      display( Context.snum, headerflag );
+			      operSetTimer();
 			    }
 			}
 		    }
@@ -3086,29 +3027,29 @@ void watch(void)
 		    cdbeep();
 		  break;
 		case '~':                 /* toggle debug display */
-		  if (CqContext.snum > 0) 
+		  if (Context.snum > 0) 
 		    {
 		      if (normal)
 			normal = FALSE;
 		      else
 			normal = TRUE;
-		      CqContext.redraw = TRUE;
+		      Context.redraw = TRUE;
 		      cdclear();
 		    }
 		  else
 		    cdbeep();
 		  break;
 		case '/':                /* ship list - dwp */
-		  stoptimer();
+		  operStopTimer();
 		  playlist( TRUE, FALSE, 0 );
-		  CqContext.redraw = TRUE;
-		  setopertimer();
+		  Context.redraw = TRUE;
+		  operSetTimer();
 		  break;
 		case '\\':               /* big ship list - dwp */
-		  stoptimer();
+		  operStopTimer();
 		  playlist( TRUE, TRUE, 0 );
-		  CqContext.redraw = TRUE;
-		  setopertimer();
+		  Context.redraw = TRUE;
+		  operSetTimer();
 		  break;
 		case '!':
 		  if (toggle_flg)
@@ -3169,28 +3110,28 @@ void watch(void)
 		      
 		      snum = i;
 			
-		      CqContext.redraw = TRUE;
+		      Context.redraw = TRUE;
 		      
 		      if (live_ships)
 			if ((snum > 0 && stillalive(snum)) || 
 			    (snum == DISPLAY_DOOMSDAY && Doomsday->status == DS_LIVE))
 			  {
-			    CqContext.snum = snum;
+			    Context.snum = snum;
 			    break;
 			  }
 			else
 			  continue;
 		      else
 			{
-			  CqContext.snum = snum;
+			  Context.snum = snum;
 			  break;
 			}
 		    }
 		  if (normal)
 		    {
-		      stoptimer();
-		      display( CqContext.snum, headerflag );
-		      setopertimer();
+		      operStopTimer();
+		      display( Context.snum, headerflag );
+		      operSetTimer();
 		    }
 		  break;
 		case '<':  /* reverse rotate ship numbers (including doomsday)  - dwp */
@@ -3247,28 +3188,28 @@ void watch(void)
 		      
 		      snum = i;
 			
-		      CqContext.redraw = TRUE;
+		      Context.redraw = TRUE;
 		      
 		      if (live_ships)
 			if ((snum > 0 && stillalive(snum)) || 
 			    (snum == DISPLAY_DOOMSDAY && Doomsday->status == DS_LIVE))
 			  {
-			    CqContext.snum = snum;
+			    Context.snum = snum;
 			    break;
 			  }
 			else
 			  continue;
 		      else
 			{
-			  CqContext.snum = snum;
+			  Context.snum = snum;
 			  break;
 			}
 		    }
 		  if (normal)
 		    {
-		      stoptimer();
-		      display( CqContext.snum, headerflag );
-		      setopertimer();
+		      operStopTimer();
+		      display( Context.snum, headerflag );
+		      operSetTimer();
 		    }
 
 		  break;
@@ -3415,7 +3356,7 @@ void dowatchhelp(void)
 
   putpmt( MTXT_DONE, MSG_LIN2 );
   cdrefresh();
-  while ( ! iogtimed( &ch, 1 ) )
+  while ( ! iogtimed( &ch, 1.0 ) )
     ;
   cdclrl( MSG_LIN2, 1 );
   
@@ -3436,7 +3377,7 @@ void toggle_line(int snum, int old_snum)
   build_toggle_str(old_snum_str,old_snum);
   
   sprintf(buf,frmt_str,snum_str, old_snum_str);
-  cdputs( buf, MSG_LIN1, (CqContext.maxcol-(strlen(buf))));  /* at end of line */
+  cdputs( buf, MSG_LIN1, (Context.maxcol-(strlen(buf))));  /* at end of line */
   
   return;
 
@@ -3610,4 +3551,155 @@ int DoInit(char InitChar, int cmdline)
   clog("OPER: %s initialized '%c'",
        operName, InitChar);
 
+  return TRUE;
 }
+
+void EnableConqoperSignalHandler(void)
+{
+#ifdef DEBUG_SIG
+  clog("EnableConquestSignalHandler() ENABLED");
+#endif
+  
+  signal(SIGHUP, (void (*)(int))DoConqoperSig);
+  signal(SIGTSTP, SIG_IGN);
+  signal(SIGTERM, (void (*)(int))DoConqoperSig);  
+  signal(SIGINT, (void (*)(int))DoConqoperSig);
+  signal(SIGQUIT, (void (*)(int))DoConqoperSig);
+
+  return;
+}
+
+void DoConqoperSig(int sig)
+{
+  
+#ifdef DEBUG_SIG
+  clog("DoSig() got SIG %d", sig);
+#endif
+  
+  switch(sig)
+    {
+    case SIGTERM:
+    case SIGINT:
+    case SIGHUP:
+    case SIGQUIT:
+      operStopTimer();
+      cdrefresh();
+      cdend();
+      exit(0);			/* WE EXIT HERE */
+      break;
+    default:
+      break;
+    }
+
+  EnableConqoperSignalHandler();	/* reset */
+  return;
+}
+
+
+/*  astoperservice - ast service routine for conqoper */
+/*  SYNOPSIS */
+/*    astservice */
+/* This routine gets called from a sys$setimr ast. Normally, it outputs */
+/* one screen update and then sets up another timer request. */
+void astoperservice(int sig)
+{
+  /* Don't do anything if we're not supposed to. */
+  if ( ! Context.display )
+    return;
+  
+  operStopTimer();
+  
+  /* Perform one ship display update. */
+  display( Context.snum, headerflag );
+  
+  /* Schedule for next time. */
+  operSetTimer();
+  
+  return;
+  
+}
+
+/*  operSetTimer - set timer to display() for conqoper...*/
+/*  SYNOPSIS */
+void operSetTimer(void)
+{
+  static struct sigaction Sig;
+  
+#ifdef HAS_SETITIMER
+  struct itimerval itimer;
+#endif
+  
+  Sig.sa_handler = (void (*)(int))astoperservice;
+  
+  Sig.sa_flags = 0;
+
+  if (sigaction(SIGALRM, &Sig, NULL) == -1)
+    {
+      clog("clntSetTimer():sigaction(): %s\n", strerror(errno));
+      exit(errno);
+    }
+  
+#ifdef HAS_SETITIMER
+
+  if (Context.updsec >= 1 && Context.updsec <= 10)
+    {
+      if (Context.updsec == 1)
+	{
+	  itimer.it_value.tv_sec = 1;
+	  itimer.it_value.tv_usec = 0;
+	}
+      else
+	{
+	  itimer.it_value.tv_sec = 0;
+	  itimer.it_value.tv_usec = (1000000 / Context.updsec);
+	}
+    }
+  else
+    {
+      itimer.it_value.tv_sec = 0;
+      itimer.it_value.tv_usec = (1000000 / 2); /* 2/sec */
+    }
+
+  itimer.it_interval.tv_sec = itimer.it_value.tv_sec;
+  itimer.it_interval.tv_usec = itimer.it_value.tv_usec;
+
+  setitimer(ITIMER_REAL, &itimer, NULL);
+#else
+  alarm(1);			/* set alarm() */
+#endif  
+  return;
+  
+}
+
+
+
+/*  operStopTimer - cancel timer */
+/*  SYNOPSIS */
+/*    operStopTimer */
+void operStopTimer(void)
+{
+#ifdef HAS_SETITIMER
+  struct itimerval itimer;
+#endif
+  
+  Context.display = FALSE;
+  
+
+  signal(SIGALRM, SIG_IGN);
+  
+#ifdef HAS_SETITIMER
+  itimer.it_value.tv_sec = itimer.it_interval.tv_sec = 0;
+  itimer.it_value.tv_usec = itimer.it_interval.tv_usec = 0;
+  
+  setitimer(ITIMER_REAL, &itimer, NULL);
+#else
+  alarm(0);
+#endif
+
+
+  Context.display = TRUE;
+  
+  return;
+  
+}
+
