@@ -2,9 +2,9 @@
 
 /************************************************************************
  *
- * $Id: conqsvr42.c,v 1.9 2001/12/26 22:06:13 jon Exp $
+ * $Id: conqsvr42.c,v 1.15 2004/01/03 09:05:02 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /*                               C O N Q V M S */
@@ -26,28 +26,39 @@
 
 #include "conqdef.h"
 #include "conqcom.h"
-#include "conqcom2.h"
+#include "context.h"
+#include "conf.h"
 #include "global.h"
+#include "record.h"
 
-
-/* int GetConquestUID(void) - return conquest's User ID */
-int GetConquestUID(void)
+#include "server.h"
+/* int GetUID(void) - return a User ID */
+int GetUID(char *name)
 {
   struct passwd *conq_pwd;
-  static int theuid;
-  
-  if ((conq_pwd = getpwnam(ROOT_USER)) == NULL)
+  char *myusername = glname();
+  char *chkname;
+
+#if defined(CYGWIN)
+  /* name root doesn't usually exist, so default to myusername */
+  name = NULL;
+#endif
+
+  if (!name)
+    chkname = myusername;
+  else
+    chkname = name;
+
+  if ((conq_pwd = getpwnam(chkname)) == NULL)
     {
-      fprintf(stderr, "conqsvr42: GetConquestUID(%s): can't get user: %s",
-	      ROOT_USER,
-	      sys_errlist[errno]);
+      fprintf(stderr, "conqsvr42: GetUID(%s): can't get user: %s\n",
+	      chkname,
+	      strerror(errno));
       
       return(ERR);
     }
   
-  theuid = conq_pwd->pw_uid;
-  
-  return(theuid);
+  return(conq_pwd->pw_uid);
 }
 
 
@@ -61,7 +72,7 @@ int GetConquestGID(void)
     {
       fprintf(stderr, "conqsvr42: GetConquestGID(%s): can't get group: %s",
 	      CONQUEST_GROUP,
-	      sys_errlist[errno]);
+	      strerror(errno));
       
       return(ERR);
     }
@@ -72,254 +83,9 @@ int GetConquestGID(void)
 }
 
 
-
-
 /*  astoff - disable asts */
 /*  SYNOPSIS */
 /*    astoff */
-void astoff(void)
-{
-  
-  /*    sys__setast( %val(0) )		/* disable asts */
-    /*  JET_ASTOFF();			/* block sigs? */
-  return;
-  
-}
-
-
-/*  aston - enable asts */
-/*  SYNOPSIS */
-/*    aston */
-void aston(void)
-{
-  
-  /*    sys$setast( %val(1) )		/* enable asts */
-    /*  JET_ASTON();			/* enable sigs? */
-  return;
-  
-}
-
-void EnableConquestSignalHandler(void)
-{
-#ifdef DEBUG_SIG
-  clog("EnableConquestSignalHandler() ENABLED");
-#endif
-  
-  signal(SIGHUP, (void (*)(int))DoConquestSig);
-  signal(SIGTSTP, SIG_IGN);
-  signal(SIGTERM, (void (*)(int))DoConquestSig);  
-  signal(SIGINT, SIG_IGN);
-
-  if (isagod(-1) || sysconf_AllowSigquit == TRUE)
-    {
-      signal(SIGQUIT, (void (*)(int))DoConquestSig);
-    }
-  else
-    {
-      signal(SIGQUIT, SIG_IGN);
-    }
-  
-  return;
-}
-
-void EnableConqoperSignalHandler(void)
-{
-#ifdef DEBUG_SIG
-  clog("EnableConquestSignalHandler() ENABLED");
-#endif
-  
-  signal(SIGHUP, (void (*)(int))DoConqoperSig);
-  signal(SIGTSTP, SIG_IGN);
-  signal(SIGTERM, (void (*)(int))DoConqoperSig);  
-  signal(SIGINT, (void (*)(int))DoConqoperSig);
-  signal(SIGQUIT, (void (*)(int))DoConqoperSig);
-
-  return;
-}
-
-void DoConquestSig(int sig)
-{
-  
-#ifdef DEBUG_SIG
-  clog("DoConquestSig() got SIG %d", sig);
-#endif
-  
-  switch(sig)
-    {
-    case SIGQUIT:
-      stoptimer();
-      drpexit();
-      cdclear();
-      cdrefresh();
-      conqstats(CqContext.snum);		/* update stats */
-				/* now we clear ship's elapsed/cpu seconds
-				   so that there won't be a huge addition to
-				   the Teams/Users/Ships timing stats when
-				   a VACANT ships re-enters Conquest */
-      Ships[CqContext.snum].ctime = 0;
-      Ships[CqContext.snum].etime = 0;
-      conqend();
-      cdend();
-      
-      exit(0);
-      break;
-
-    case SIGINT:
-    case SIGTERM:
-    case SIGHUP:
-      cdend();
-      exit(0);
-      break;
-    default:
-      break;
-    }
-
-  EnableConquestSignalHandler();	/* reset */
-  return;
-}
-
-void DoConqoperSig(int sig)
-{
-  
-#ifdef DEBUG_SIG
-  clog("DoSig() got SIG %d", sig);
-#endif
-  
-  switch(sig)
-    {
-    case SIGTERM:
-    case SIGINT:
-    case SIGHUP:
-    case SIGQUIT:
-      stoptimer();
-      cdrefresh();
-      conqend();
-      cdend();
-      exit(0);			/* WE EXIT HERE */
-      break;
-    default:
-      break;
-    }
-
-  EnableConqoperSignalHandler();	/* reset */
-  return;
-}
-
-
-/*  astservice - ast service routine for conquest */
-/*  SYNOPSIS */
-/*    astservice */
-/* This routine gets called from a sys$setimr ast. Normally, it outputs */
-/* one screen update and then sets up another timer request. */
-void astservice(int sig)
-{
-  int now;
-  int readone;
-  static int RMsggrand = 0;
-  int difftime;
-
-  /* Don't do anything if we're not supposed to. */
-  if ( ! CqContext.display )
-    return;
-  
-  /* Don't do anything if we're dead. */
-  if ( ! stillalive( CqContext.snum ) )
-    return;
-  stoptimer();
-  drcheck();				/* handle driver logic */
-  
-  /* See if we can display a new message. */
-
-				/* for people with 25 lines, we
-				   use a different timer so that
-				   NEWMSG_GRAND intervals will determine
-				   whether it's time to display a new
-				   message... Otherwise, CqContext.msgrand
-				   is used - which means that NEWMSG_GRAND
-				   interval will have to pass after issuing
-				   any command before a new msg will disp
-				   12/28/98 */
-  readone = FALSE;
-  if ( CqContext.msgok )
-    {
-      /*
-      clog("### RMsggrand = %d, CqContext.msgrand = %d", RMsggrand, CqContext.msgrand);
-      */
-
-      if (RMsg_Line != MSG_LIN1)
-	{			/* we have line 25 for msgs */
-	  difftime = dgrand( RMsggrand, &now );
-	}
-      else
-	{
-	  difftime = dgrand( CqContext.msgrand, &now );
-	}
-
-      /*
-	clog("difftime = %d, RMsg_Line = %d, MSG_LIN1 = %d\n\tNEWMSG_GRAND = %d, now = %d", difftime, RMsg_Line, MSG_LIN1, NEWMSG_GRAND, now);
-       */
-
-      if ( difftime >= NEWMSG_GRAND )
-	if ( getamsg( CqContext.snum, &Ships[CqContext.snum].lastmsg ) )
-	  {
-	    if (readmsg( CqContext.snum, Ships[CqContext.snum].lastmsg, 
-			 RMsg_Line ) == TRUE)
-	      {
-		if (Msgs[Ships[CqContext.snum].lastmsg].msgfrom != 
-		    CqContext.snum)
-		  if (conf_MessageBell == TRUE)
-		    cdbeep();
-				/* set both timers, regardless of which
-				   one we're actally concerned with */
-		CqContext.msgrand = now;
-		RMsggrand = now;
-		readone = TRUE;
-	      }
-	  }
-    }
-
-  /* Perform one ship display update. */
-  display( CqContext.snum, FALSE );
-  
-  
-  /* Un-read the message if there's a chance it got garbaged. */
-  /* JET 3/24/96 - another try with curses timer disabled */
-  if ( readone )
-    if (RMsg_Line == MSG_LIN1)	/* we don't have an extra msg line */
-      if ( iochav() )
-	Ships[CqContext.snum].lastmsg = modp1( Ships[CqContext.snum].lastmsg - 1, MAXMESSAGES );
-  
-  /* Schedule for next time. */
-  settimer();
-  
-  return;
-  
-}
-
-/*  astoperservice - ast service routine for conqoper */
-/*  SYNOPSIS */
-/*    astservice */
-/* This routine gets called from a sys$setimr ast. Normally, it outputs */
-/* one screen update and then sets up another timer request. */
-void astoperservice(int sig)
-{
-  /* Don't do anything if we're not supposed to. */
-  if ( ! CqContext.display )
-    return;
-  
-  stoptimer();
-  
-  /* Perform one ship display update. */
-  display( CqContext.snum, headerflag );
-  
-  /* Schedule for next time. */
-  setopertimer();
-  
-  return;
-  
-}
-
-
 /*  comsize - return size of the common block (in bytes) */
 /*  SYNOPSIS */
 /*    int size */
@@ -341,26 +107,6 @@ void comsize( unsigned long *size )
 }
 
 
-/*  conqend - machine dependent clean-up */
-/*  SYNOPSIS */
-/*    conqend */
-void conqend(void)
-{
-
-  char msgbuf[128];
-
-  if (CqContext.entship == TRUE)
-    {				/* let everyone know we're leaving */
-      sprintf(msgbuf, "%s has left the game.",
-	      Users[CqContext.unum].alias);
-      stormsg(MSG_COMP, MSG_ALL, msgbuf);
-    }
-  
-  return;
-  
-}
-
-
 /*  conqinit - machine dependent initialization */
 /*  SYNOPSIS */
 /*    conqinit */
@@ -378,7 +124,7 @@ void conqinit(void)
     {
       clog("conqinit(): nice(CONQUEST_PRI (%d)): failed: %s",
 	   CONQUEST_PRI,
-	   sys_errlist[errno]);
+	   strerror(errno));
     }
 
 #if defined(DEBUG_FLOW)
@@ -390,22 +136,19 @@ void conqinit(void)
   
   /* Set up game environment. */
   
-  /* Figure out which gamcron file to use (and if we're gonna use one). */
-  
   /* Other house keeping. */
-  CqContext.pid = getpid();		
-  CqContext.hasnewsfile = ( strcmp( C_CONQ_NEWSFILE, "" ) != 0 );
+  Context.pid = getpid();		
+  Context.hasnewsfile = ( strcmp( C_CONQ_NEWSFILE, "" ) != 0 );
   
   /* Zero process id of our child (since we don't have one yet). */
-  CqContext.childpid = 0;
+  Context.childpid = 0;
   
   /* Zero last time drcheck() was called. */
-  CqContext.drchklastime = 0;
+  Context.drchklastime = 0;
   
   /* Haven't scanned anything yet. */
-  CqContext.lastinfostr[0] = EOS;
+  Context.lastinfostr[0] = EOS;
 
-  
   return;
   
 }
@@ -439,12 +182,12 @@ void conqstats( int snum )
 				/* update elapsed time in History[] 
 				   for this user */
 
-      if (CqContext.histslot != ERR && History[CqContext.histslot].histunum == unum)
+      if (Context.histslot != ERR && History[Context.histslot].histunum == unum)
 	{
-	  difftime = getnow(NULL, 0) - History[CqContext.histslot].histlog;
+	  difftime = getnow(NULL, 0) - History[Context.histslot].histlog;
 	  if (difftime < (time_t)0)
 	    difftime = (time_t)0;
-	  History[CqContext.histslot].elapsed = difftime;
+	  History[Context.histslot].elapsed = difftime;
 	}
 
       team = Users[unum].team;
@@ -472,16 +215,16 @@ void drcheck(void)
   
   /* If we haven't been getting cpu time in recent history, do no-thing. */
   /*  gsecs(playtime);*/
-  if ( dsecs( CqContext.drchklastime, &CqContext.drchklastime ) > TIMEOUT_DRCHECK )
+  if ( dsecs( Context.drchklastime, &Context.drchklastime ) > TIMEOUT_DRCHECK )
     return;
   
   if ( dsecs( Driver->drivtime, &(Driver->playtime) ) > TIMEOUT_DRIVER )
     {
-      if ( CqContext.childpid != 0 )
+      if ( Context.childpid != 0 )
 	{
 	  /* We own the driver. See if it's still there. */
-	  ppid = CqContext.childpid;
-	  if ( kill(CqContext.childpid, 0) != -1 )
+	  ppid = Context.childpid;
+	  if ( kill(Context.childpid, 0) != -1 )
 	    {
 	      /* He's still alive and belongs to us. */
 	      gsecs( &(Driver->drivtime) );
@@ -491,7 +234,7 @@ void drcheck(void)
 	    clog( "drcheck(): Wrong ppid %d.", ppid );
 	  
 	  /* If we got here, something was wrong; disown the child. */
-	  CqContext.childpid = 0;
+	  Context.childpid = 0;
 	}
       
       PVLOCK(&ConqInfo->lockword);
@@ -528,22 +271,22 @@ void drcreate(void)
   if ((pid = fork()) == -1)
     {				/* error */
       Driver->drivstat = DRS_OFF;
-      clog( "drcreate(): fork(): %s", sys_errlist[errno]);
+      clog( "drcreate(): fork(): %s", strerror(errno));
       return;
     }
   
   if (pid == 0)
     {				/* The child: aka "The Driver" */
-      sprintf(drivcmd, "%s/%s", CONQHOME, C_CONQ_CONQDRIV);
+      sprintf(drivcmd, "%s/%s", CONQLIBEXEC, C_CONQ_CONQDRIV);
       execl(drivcmd, drivcmd, NULL);
-      clog("drcreate(): exec(): %s", sys_errlist[errno]);
+      clog("drcreate(): exec(): %s", strerror(errno));
       perror("exec");		/* shouldn't be reached */
       exit(1);
       /* NOTREACHED */
     }
   else
     {				/* We're the parent, store pid */
-      CqContext.childpid = pid;	
+      Context.childpid = pid;	
     }
   
   return;
@@ -556,11 +299,11 @@ void drcreate(void)
 /*    drkill */
 void drkill(void)
 {
-  if ( CqContext.childpid != 0 )
-    if ( CqContext.childpid == Driver->drivpid && Driver->drivstat == DRS_RUNNING )
+  if ( Context.childpid != 0 )
+    if ( Context.childpid == Driver->drivpid && Driver->drivstat == DRS_RUNNING )
       {
 	PVLOCK(&ConqInfo->lockword);
-	if ( CqContext.childpid == Driver->drivpid && Driver->drivstat == DRS_RUNNING )
+	if ( Context.childpid == Driver->drivpid && Driver->drivstat == DRS_RUNNING )
 	  Driver->drivstat = DRS_KAMIKAZE;
 	PVUNLOCK(&ConqInfo->lockword);
       }
@@ -578,14 +321,14 @@ void drpexit(void)
   
   int i;
   
-  if ( CqContext.childpid != 0 )
+  if ( Context.childpid != 0 )
     {
       /* We may well have started the driver. */
       drkill();
-      for ( i = 1; CqContext.childpid == Driver->drivpid && i <= 50; i = i + 1 )
+      for ( i = 1; Context.childpid == Driver->drivpid && i <= 50; i = i + 1 )
 	c_sleep( 0.1 );
-      if ( CqContext.childpid == Driver->drivpid )
-	clog("drpexit(): Driver didn't exit; pid = %08x", CqContext.childpid );
+      if ( Context.childpid == Driver->drivpid )
+	clog("drpexit(): Driver didn't exit; pid = %08x", Context.childpid );
     }
   
   return;
@@ -643,7 +386,7 @@ void helplesson(void)
   char buf[MSGMAXLINE];
   char helpfile[BUFFER_SIZE];
   
-  sprintf(helpfile, "%s/%s", CONQHOME, C_CONQ_HELPFILE);
+  sprintf(helpfile, "%s/%s", CONQSHARE, C_CONQ_HELPFILE);
   sprintf( buf, "%s: Can't open.", helpfile );
   pagefile( helpfile, buf);
   
@@ -667,10 +410,32 @@ void initstats( int *ctemp, int *etemp )
 }
 
 
-/*  isagod - determine if a user is a god or not - NULL means current user */
+/*  isagod - determine if a user is a god (oper) or not */
+/*   if a valid user is passed, then the OOPT_OPER priviledge is */
+/*    checked.  Otherwise, the current user must be a member of */
+/*    the 'conquest' group (conqoper, conqdriv, etc */
 /*  SYNOPSIS */
 /*    int flag, isagod */
 /*    flag = isagod() */
+
+/* For cygwin, everybody is a god for non-user num checks. */
+#if defined(CYGWIN)
+int isagod( int unum )
+{
+  if (unum == -1)               /* get god status for current user */
+    {
+      return TRUE;
+    }
+  else
+    {				/* else a user number passed in */
+				/* just check for OOPT_OPER */
+      if (Users[unum].ooptions[OOPT_OPER])
+	return TRUE;
+      else
+	return FALSE;
+    }
+}
+#else /* !CGYWIN */
 int isagod( int unum )
 {
   static struct group *grp = NULL;
@@ -681,16 +446,17 @@ int isagod( int unum )
   god = FALSE;
   
   if (unum == -1)		/* get god status for current user */
-    {				/* now find out whether we're in it */
-      strncpy(myname, cuserid(NULL), BUFFER_SIZE - 2);
+    {
+      strncpy(myname, glname(), BUFFER_SIZE);
       myname[BUFFER_SIZE - 1] = EOS;
     }
   else
     {				/* else a user number passed in */
-				/* only locals can be godlike */
-      if (Users[unum].type != UT_LOCAL)
-	return(FALSE);
-      strcpy(myname, Users[unum].username);
+				/* just check for OOPT_OPER */
+      if (Users[unum].ooptions[OOPT_OPER])
+	return TRUE;
+      else
+	return FALSE;
     }
   
   if (grp == NULL)
@@ -702,7 +468,7 @@ int isagod( int unum )
 	  clog("isagod(%s): getgrnam(%s) failed: %s",
 	       myname,
 	       CONQUEST_GROUP,
-	       sys_errlist[errno]);
+	       strerror(errno));
 	  
 	  god = FALSE;
 	  return(FALSE);
@@ -735,6 +501,7 @@ int isagod( int unum )
   
 }
 
+#endif /* !CYGWIN */
 
 /*  news - list current happenings */
 /*  SYNOPSIS */
@@ -742,9 +509,8 @@ int isagod( int unum )
 void news(void)
 {
   char newsfile[BUFFER_SIZE];
-  extern char *c_conq_newsfile;
   
-  sprintf(newsfile, "%s/%s", CONQHOME, C_CONQ_NEWSFILE);
+  sprintf(newsfile, "%s/%s", CONQSHARE, C_CONQ_NEWSFILE);
   
   pagefile( newsfile, "No news is good news.");
   
@@ -753,140 +519,7 @@ void news(void)
 }
 
 
-/*  settimer - set timer to display() */
-/*  SYNOPSIS */
-/*    csetimer */
-void settimer(void)
-{
-  static struct sigaction Sig;
-  
-#ifdef HAS_SETITIMER
-  struct itimerval itimer;
-#endif
-  
-  Sig.sa_handler = (void (*)(int))astservice;
-  
-/*  Sig.sa_flags = SA_RESTART;*/	/* restart syscalls! */
 
-  Sig.sa_flags = 0;
-
-  if (sigaction(SIGALRM, &Sig, NULL) == -1)
-    {
-      clog("settimer():sigaction(): %s\n", sys_errlist[errno]);
-      exit(errno);
-    }
-  
-#ifdef HAS_SETITIMER
-
-  if (sysconf_AllowFastUpdate == TRUE && conf_DoFastUpdate == TRUE)
-    {
-      
-      /* 2 updates per sec */
-      itimer.it_value.tv_sec = 0;
-      itimer.it_value.tv_usec = 500000;
-      
-      itimer.it_interval.tv_sec = 0;
-      itimer.it_interval.tv_usec = 500000;
-    }
-  else
-    {
-
-      /* 1 update per second */
-      itimer.it_value.tv_sec = 1;
-      itimer.it_value.tv_usec = 0;
-      
-      itimer.it_interval.tv_sec = 1;
-      itimer.it_interval.tv_usec = 0;
-    }
-
-  setitimer(ITIMER_REAL, &itimer, NULL);
-#else
-  alarm(1);			/* set alarm() */
-#endif  
-  return;
-  
-}
-
-
-/*  setopertimer - set timer to display() for conqoper...*/
-/*  SYNOPSIS */
-void setopertimer(void)
-{
-  static struct sigaction Sig;
-  
-#ifdef HAS_SETITIMER
-  struct itimerval itimer;
-#endif
-  
-  Sig.sa_handler = (void (*)(int))astoperservice;
-  
-  Sig.sa_flags = 0;
-
-  if (sigaction(SIGALRM, &Sig, NULL) == -1)
-    {
-      clog("settimer():sigaction(): %s\n", sys_errlist[errno]);
-      exit(errno);
-    }
-  
-#ifdef HAS_SETITIMER
-
-  if (sysconf_AllowFastUpdate == TRUE && conf_DoFastUpdate == TRUE)
-    {
-      
-      /* 2 updates per sec */
-      itimer.it_value.tv_sec = 0;
-      itimer.it_value.tv_usec = 500000;
-      
-      itimer.it_interval.tv_sec = 0;
-      itimer.it_interval.tv_usec = 500000;
-    }
-  else
-    {
-
-      /* 1 update per second */
-      itimer.it_value.tv_sec = 1;
-      itimer.it_value.tv_usec = 0;
-      
-      itimer.it_interval.tv_sec = 1;
-      itimer.it_interval.tv_usec = 0;
-    }
-
-  setitimer(ITIMER_REAL, &itimer, NULL);
-#else
-  alarm(1);			/* set alarm() */
-#endif  
-  return;
-  
-}
-
-
-/*  stoptimer - cancel timer */
-/*  SYNOPSIS */
-/*    stoptimer */
-void stoptimer(void)
-{
-#ifdef HAS_SETITIMER
-  struct itimerval itimer;
-#endif
-  
-  CqContext.display = FALSE;
-  
-  signal(SIGALRM, SIG_IGN);
-  
-#ifdef HAS_SETITIMER
-  itimer.it_value.tv_sec = itimer.it_interval.tv_sec = 0;
-  itimer.it_value.tv_usec = itimer.it_interval.tv_usec = 0;
-  
-  setitimer(ITIMER_REAL, &itimer, NULL);
-#else
-  alarm(0);
-#endif
-  
-  CqContext.display = TRUE;
-  
-  return;
-  
-}
 
 
 /*  upchuck - update the common block to disk. */
