@@ -2,9 +2,9 @@
 
 /************************************************************************
  *
- * $Id: display.c,v 1.6 1999/02/06 22:19:15 jon Exp $
+ * $Id: display.c,v 1.9 2003/11/02 20:53:00 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /**********************************************************************/
@@ -17,10 +17,12 @@
 
 #include "conqdef.h"
 #include "conqcom.h"
-#include "conqcom2.h"
+#include "context.h"
+#include "conf.h"
 #include "global.h"
 #include "color.h"
 
+#include "record.h"
 
 #define GREEN_ALERT 0
 #define YELLOW_ALERT 1
@@ -35,25 +37,82 @@ static int AlertLevel = GREEN_ALERT;
 extern real LastPhasDist;	/* defined in conqlb.c */
 
 
-void do_bottomborder(void)
+void do_bottomborder(int snum, char *buf, int attrib, int bufattr)
 {
-  int lin;
+  int lin, col;
 
   lin = DISPLAY_LINS + 1;
 
-  cdline( lin, 1, lin, CqContext.maxcol );
+  /* draw the line */
+  attrset(attrib);
+  cdline( lin, 1, lin, Context.maxcol );
   mvaddch(lin - 1, STAT_COLS - 1, ACS_BTEE);
+  attrset(0);
+
+  /* now the buffer, if there */
+  if ( buf && buf[0] != EOS )
+    {
+      if (UserConf.AltHUD)
+	col = Context.maxcol - (strlen(buf) + 1);
+      else
+	col = (int) (Context.maxcol - STAT_COLS - strlen(buf)) / 
+	  (int)2 + STAT_COLS + 1;
+
+      if (Context.hascolor)
+	{
+	  attrset(bufattr);
+	}
+      else
+	{
+	  if (bufattr == COLOR_PAIR(COL_REDBLACK))
+	    attrset(A_BOLD | A_BLINK);
+	  else if (bufattr == COLOR_PAIR(COL_YELLOWBLACK))
+	    attrset(A_BOLD);
+	}
+      cdputs( buf, DISPLAY_LINS+1, col );
+      attrset(0);
+    }
+
+  /* if alt hud and we are a ship... */
+  if (UserConf.AltHUD && snum > 0 && 
+      snum <= MAXSHIPS)
+    {				/* some additional info */
+      /* current firing angle */
+      col = STAT_COLS + 1;
+
+      cprintf(lin, col, ALIGN_LEFT, "#%d#FA:#%d#%3d",
+	      LabelColor,
+	      InfoColor,
+	      (int)Ships[snum].lastblast);
+      col += 7;
+
+      /* last target (I)nfo info */
+      cprintf(lin, col, ALIGN_LEFT, "#%d#TA/D:#%d#%3s#%d#:#%d#%3d#%d#/#%d#%d",
+	      LabelColor,
+	      InfoColor,
+	      Context.lasttarg,
+	      LabelColor,
+	      InfoColor,
+	      Context.lasttang,
+	      LabelColor,
+	      InfoColor,
+	      Context.lasttdist);
+    }
+
+
+  return;
 }
 
-void do_border(void)
+void do_border(int snum, int attr)
 {
   int lin;
   
   lin = DISPLAY_LINS + 1;
   
-  
+  attrset(attr);
   cdline( 1, STAT_COLS, lin, STAT_COLS );
-  do_bottomborder();
+  attrset(0);
+  do_bottomborder(snum, NULL, attr, 0);
   
   return;
 }
@@ -81,15 +140,9 @@ int alertcolor(int alert)
   return(theattrib);
 }
 
-void draw_alertborder(int alert)
+void draw_alertborder(int snum, int alert)
 {
-  
-  attrset(alertcolor(alert));
-  do_border();
-  attrset(0);
-  
-  /*  cdrefresh(); */
-  
+  do_border(snum, alertcolor(alert));
   return;
 }
 
@@ -125,15 +178,15 @@ void display( int snum, int display_info )
   static char zbuf[MSGMAXLINE];
   static char *dstatstr;
 
-  if ( CqContext.redraw )
+  if ( Context.redraw )
     {
       cdclear();
       lin = DISPLAY_LINS + 1;
-      draw_alertborder(AlertLevel);
+      draw_alertborder(snum, AlertLevel);
     }
   else
     {
-      cdclra( 1, STAT_COLS  + 1, DISPLAY_LINS, CqContext.maxcol + 1 );
+      cdclra( 1, STAT_COLS  + 1, DISPLAY_LINS, Context.maxcol + 1 );
     }
 
   dobeep = FALSE;
@@ -143,15 +196,15 @@ void display( int snum, int display_info )
   minsenemy = 0;
 
   if (snum > 0)
-    lsmap = Ships[snum].map;
+    lsmap = SMAP(snum);
   else
-    lsmap = FALSE;
+    lsmap = 0;
   
   if ( lsmap )
     {
       scale = MAP_FAC;
       
-      if (sysconf_DoLocalLRScan)
+      if (UserConf.DoLocalLRScan)
 	{
 	  cenx = Ships[snum].x;
 	  ceny = Ships[snum].y;
@@ -226,7 +279,7 @@ void display( int snum, int display_info )
 		ch = Teams[Planets[i].team].torpchar;
 	      
 	      /* Display the planet; either it's numeric or it's not. */
-	      if ( Ships[snum].options[OPT_NUMERICMAP] )
+	      if ( UserConf.DoNumMap )
 		{
 		  sprintf( buf, "%d", Planets[i].armies);
 		  l = strlen(buf);
@@ -274,7 +327,7 @@ void display( int snum, int display_info )
 	  
 	  /* If necessary, display the planet name. */
 	  if ( snum < 0 || (snum > 0 && 
-			    Ships[snum].options[OPT_PLANETNAMES]) ) /* dwp */
+			    UserConf.ShowPlanNames) ) /* dwp */
 	    {
 	      sprintf(buf, "%c%c%c", Planets[i].name[0], Planets[i].name[1], Planets[i].name[2]);
 	      attrset(palertcol);
@@ -304,7 +357,7 @@ void display( int snum, int display_info )
 		  cdput( ConqInfo->chrplanets[Planets[i].type], lin, col + 1);
 		  attrset(0);
 		}
-	      if ( snum < 0 || (snum > 0 && Ships[snum].options[OPT_PLANETNAMES]) ) /* dwp */
+	      if ( snum < 0 || (snum > 0 && UserConf.ShowPlanNames) ) /* dwp */
 		if ( (lin + 1 <= DISPLAY_LINS) && col + 1< cdcols() )
 		  {
 		    attrset(palertcol);
@@ -345,7 +398,7 @@ void display( int snum, int display_info )
     {
       /* Display phaser graphics. */
       if ( ! lsmap && Ships[snum].pfuse > 0 )
-	if ( Ships[snum].options[ OPT_PHASERGRAPHICS] )
+	if ( UserConf.ShowPhasers )
 	  {
 	    sd = sind(Ships[snum].lastphase);
 	    cd = cosd(Ships[snum].lastphase);
@@ -364,7 +417,7 @@ void display( int snum, int display_info )
   for ( i = 1; i <= MAXSHIPS; i = i + 1 )
     if ( Ships[i].status != SS_OFF )
       {
-	if (sysconf_DoLRTorpScan)
+	if (UserConf.DoLRTorpScan)
 	  {
 	    /* Display the torps on a LR scan if it's a friend. */
 	    if (lsmap)
@@ -391,7 +444,7 @@ void display( int snum, int display_info )
 	if ( ! lsmap )
 	  {
 	    /* First display exploding torps. */
-	    if ( snum < 0 || (snum > 0 && Ships[snum].options[ OPT_EXPLOSIONS]) ) /* dwp */
+	    if ( snum < 0 || (snum > 0 && UserConf.DoExplode) ) /* dwp */
 	      for ( j = 0; j < MAXTORPS; j = j + 1 )
 		if ( Ships[i].torps[j].status == TS_FIREBALL )
 		  if ( cvtcoords( cenx, ceny, Ships[i].torps[j].x, Ships[i].torps[j].y,
@@ -440,9 +493,6 @@ void display( int snum, int display_info )
 		dis = (real) dist(Ships[snum].x, Ships[snum].y, Ships[i].x, Ships[i].y );
 	    
 		/* Here's where ship to ship accurate information is "gathered". */
-		if ( dis < ACCINFO_DIST && ! Ships[i].cloaked && ! selfwar( snum ) )
-		  Ships[i].scanned[Ships[snum].team] = SCANNED_FUSE;
-	    
 		/* Check for nearest enemy and nearest scanned enemy. */
 		if ( satwar( i, snum ) )
 		  if ( i != snum )
@@ -452,7 +502,7 @@ void display( int snum, int display_info )
 		      /* 1/6/94 */
 		      /* we want any cloaked ship at warp 0.0 */
 		      /* to be invisible. */
-		      if (Ships[i].cloaked && Ships[i].warp == 0.0)
+		      if (SCLOAKED(i) && Ships[i].warp == 0.0)
 			{
 			  /* skip to next ship. this one isn't here */
 			  /* ;-} */
@@ -479,7 +529,7 @@ void display( int snum, int display_info )
 	      }	/* if a ship view (snum > 0) */
 
 	    /* There is potential for un-cloaked ships and ourselves. */
-	    if ( ! Ships[i].cloaked || i == snum )
+	    if ( ! SCLOAKED(i) || i == snum )
 	      {
 		/* ... especially if he's in the bounds of our current */
 		/*  display (either tactical or strategic map) */
@@ -488,7 +538,7 @@ void display( int snum, int display_info )
 		  {
 		    /* He's on the screen. */
 		    /* We can see him if one of the following is true: */
-#
+
 		    /*  - We are not looking at our strategic map */
 		    /*  - We're mutually at peace */
 		    /*  - Our team has scanned him and we're not self-war */
@@ -500,7 +550,7 @@ void display( int snum, int display_info )
 			  !selfwar(snum) ) ||
 			( dis <= ACCINFO_DIST ) )
 		      {
-			if ( snum > 0 && ( i == snum ) && Ships[snum].cloaked )
+			if ( snum > 0 && ( i == snum ) && SCLOAKED(snum) )
 			  ch = CHAR_CLOAKED;
 			else
 			  ch = Teams[Ships[i].team].teamchar;
@@ -540,7 +590,7 @@ void display( int snum, int display_info )
 
 			attrset(InfoColor);
 			if ( j >= 0 && j < DISPLAY_LINS && 
-			    k > STAT_COLS && k < CqContext.maxcol )
+			    k > STAT_COLS && k < Context.maxcol )
 			  cdput( dirch[idx], j, k );
 			attrset(0);
 		      }
@@ -555,7 +605,7 @@ void display( int snum, int display_info )
 		    if ( col < STAT_COLS + 1 )
 		      col = STAT_COLS + 1;
 		    else
-		      col = min( col, CqContext.maxcol );
+		      col = min( col, Context.maxcol );
 		    cdmove( lin, col );
 		  }
 	      }
@@ -563,13 +613,13 @@ void display( int snum, int display_info )
       } /* for each ship */
   
   /* Construct alert status line. */
-  if ( CqContext.redraw )
+  if ( Context.redraw )
     zzbuf[0] = EOS;
   buf[0] = EOS;
 
   if (snum > 0)
     {				/* if a ship view */
-      if ( minenemy != 0 || Ships[snum].talert )
+      if ( minenemy != 0 || STALERT(snum) )
 	{
 	  if ( mindis <= PHASER_DIST )
 	    {
@@ -587,7 +637,7 @@ void display( int snum, int display_info )
 	      c_strcpy( "Alert ", buf );
 	      dobeep = TRUE;
 	    }
-	  else if ( Ships[snum].talert )
+	  else if ( STALERT(snum) )
 	    {
 	      /* Nearby torpedos. */
 	      outattr = COLOR_PAIR(COL_YELLOWBLACK);
@@ -621,7 +671,7 @@ void display( int snum, int display_info )
 	  if ( minenemy != 0 )
 	    {
 	      appship( minenemy, buf );
-	      if ( Ships[minenemy].cloaked )
+	      if ( SCLOAKED(minenemy) )
 		appstr( " (CLOAKED)", buf );
 	    }
 	}
@@ -633,34 +683,14 @@ void display( int snum, int display_info )
 
   if (OldAlert != AlertLevel)
     {
-      draw_alertborder(AlertLevel);
+      draw_alertborder(snum, AlertLevel);
       OldAlert = AlertLevel;
     }
   
-  if ( strcmp( buf, zzbuf ) != 0 )
+  if ( (strcmp( buf, zzbuf ) != 0) || UserConf.AltHUD)
     {
       lin = DISPLAY_LINS + 1;
-      attrset(alertcolor(AlertLevel));
-      do_bottomborder();
-      attrset(0);
-
-      if ( buf[0] != EOS )
-	{
-	  col = (int) (CqContext.maxcol - STAT_COLS - strlen(buf)) / (int)2 + STAT_COLS + 1;
-	  if (HasColors)
-	    {
-	      attrset(outattr);
-	    }
-	  else
-	    {
-	      if (outattr == COLOR_PAIR(COL_REDBLACK))
-		attrset(A_BOLD | A_BLINK);
-	      else if (outattr == COLOR_PAIR(COL_YELLOWBLACK))
-		attrset(A_BOLD);
-	    }
-	  cdputs( buf, DISPLAY_LINS+1, col );
-	  attrset(0);
-	}
+      do_bottomborder(snum, buf, alertcolor(AlertLevel), outattr);
       c_strcpy( buf, zzbuf );
     }
   
@@ -676,13 +706,13 @@ void display( int snum, int display_info )
       dobeep = TRUE;
     prevsh = Ships[snum].shields;
     
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	zzsshields = -9;
 	zzcshields = ' ';
       }
     i = round( Ships[snum].shields );
-    if ( ! Ships[snum].shup || Ships[snum].rmode )
+    if ( ! SSHUP(snum) || SREPAIR(snum) )
       i = -1;
     if ( i != zzsshields || i == -1)
       {
@@ -733,7 +763,7 @@ void display( int snum, int display_info )
     
     /* Kills. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	attrset(LabelColor);
 	cdputs( "kills =", lin, col );
@@ -755,7 +785,7 @@ void display( int snum, int display_info )
     
     /* Warp. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	attrset(LabelColor);
 	cdputs( "warp =", lin, col );
@@ -782,7 +812,7 @@ void display( int snum, int display_info )
     
     /* Heading. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	attrset(LabelColor);
 	cdputs( "heading =", lin, col );
@@ -808,7 +838,7 @@ void display( int snum, int display_info )
     
     /* Fuel. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	zzsfuel = -99;
 	zzcfuel = ' ';
@@ -851,7 +881,7 @@ void display( int snum, int display_info )
     
     /* Allocation. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	attrset(LabelColor);
 	cdputs( "w/e =", lin, col );
@@ -886,7 +916,7 @@ void display( int snum, int display_info )
     
     /* Temperature. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	zzswtemp = 0;
 	zzsetemp = 0;
@@ -982,7 +1012,7 @@ void display( int snum, int display_info )
     prevdam = Ships[snum].damage;
     
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       {
 	zzsdamage = -9;
 	zzcdamage = ' ';
@@ -1008,7 +1038,7 @@ void display( int snum, int display_info )
 	zzsdamage = i;
       }
   
-    if ( Ships[snum].rmode )
+    if ( SREPAIR(snum) )
       j = 'r';
     else if ( i >= 50 )
       j = 'D';
@@ -1041,7 +1071,7 @@ void display( int snum, int display_info )
   
     /* Armies. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       zzsarmies = -666;
     i = Ships[snum].armies;
     if ( i == 0 )
@@ -1070,7 +1100,7 @@ void display( int snum, int display_info )
   
     /* Tractor beams. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       zzstowedby = 0;
     i = Ships[snum].towedby;
     if ( i == 0 )
@@ -1098,9 +1128,9 @@ void display( int snum, int display_info )
   
     /* Self destruct fuse. */
     lin = lin + 2;
-    if ( CqContext.redraw )
+    if ( Context.redraw )
       zzssdfuse = -9;
-    if ( Ships[snum].cloaked )
+    if ( SCLOAKED(snum) )
       i = -1;
     else
       i = max( 0, Ships[snum].sdfuse );
@@ -1126,7 +1156,7 @@ void display( int snum, int display_info )
       }
   
     if ( dobeep )
-      if ( Ships[snum].options[OPT_ALARMBELL] )
+      if ( UserConf.DoAlarms && !SROBOT(snum))
 	cdbeep();
   
   } /* end of ship stats display */
@@ -1152,7 +1182,7 @@ void display( int snum, int display_info )
 	  else
 	    appstr( "  (unknown)", buf);
 	  cdputs( buf, 1, STAT_COLS + 
-		  (int)(CqContext.maxcol - STAT_COLS - (strlen(buf))) / (int)2 + 1);
+		  (int)(Context.maxcol - STAT_COLS - (strlen(buf))) / (int)2 + 1);
       
 	  lin = lin + 2;
 	  dcol = col + 11;
@@ -1263,8 +1293,8 @@ void display( int snum, int display_info )
       display_headers(snum);
 
   cdrefresh();
-  CqContext.redraw = FALSE;
-  
+  Context.redraw = FALSE;
+
   return;
   
 }
@@ -1294,10 +1324,10 @@ void display_headers(int snum)
 	      Ships[snum].alias, ssbuf); 
       attrset(A_BOLD);
       cdputs( hbuf, 1, STAT_COLS + 
-	      (int)(CqContext.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
+	      (int)(Context.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
       attrset(0);
     }
-  else if ( Ships[snum].robot )
+  else if ( SROBOT(snum) )
     {
       if (ConqInfo->externrobots == TRUE) 
 	{
@@ -1306,7 +1336,7 @@ void display_headers(int snum)
 		  Ships[snum].alias, ssbuf); 
 	  attrset(A_BOLD);
 	  cdputs( hbuf, 1, STAT_COLS + 
-		  (int)(CqContext.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
+		  (int)(Context.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
 	  attrset(0);
 	}
       else 
@@ -1316,7 +1346,7 @@ void display_headers(int snum)
 		  Ships[snum].alias, ssbuf);
 	  attrset(A_BOLD);
 	  cdputs( hbuf, 1, STAT_COLS + 
-		  (int)(CqContext.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
+		  (int)(Context.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
 	  attrset(0);
 	}
     }
@@ -1327,7 +1357,7 @@ void display_headers(int snum)
 	      Ships[snum].alias, ssbuf);
       attrset(A_BOLD);
       cdputs( hbuf, 1, STAT_COLS + 
-	      (int)(CqContext.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
+	      (int)(Context.maxcol - STAT_COLS - (strlen(hbuf))) / (int)2 + 1);
       attrset(0);
     }
   
