@@ -1,8 +1,8 @@
 /************************************************************************
  *
- * $Id: ibuf.h,v 1.4 1999/02/06 22:19:17 jon Exp $
+ * $Id: ibuf.h,v 1.5 2003/11/02 20:53:00 jon Exp $
  *
- * Copyright 1999 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
+ * Copyright 1999-2004 Jon Trulson under the ARTISTIC LICENSE. (See LICENSE).
  ***********************************************************************/
 
 /**********************************************************************/
@@ -16,9 +16,13 @@
 #ifndef IBUF_H
 #define IBUF_H
 
-/* Input Buffer (ibuf.c) variables */
-#define    IBUFMAX 128
-static int iBufOffset;
-static char iBuffer[IBUFMAX];
+#define    IBUFMAX 1024
+
+void iBufInit(void);
+int iBufCount(void);
+void iBufPut(char *thestr);
+void iBufPutc(unsigned int thechar);
+unsigned int iBufGetCh(void);
+int DoMacro(int ch);
 
 #endif
